@@ -83,28 +83,34 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     );
 
 /* GstCmmlDec */
-GST_BOILERPLATE (GstCmmlDec, gst_cmml_dec, GstElement, GST_TYPE_ELEMENT);
+#define gst_cmml_dec_parent_class parent_class
+G_DEFINE_TYPE (GstCmmlDec, gst_cmml_dec, GST_TYPE_ELEMENT);
+
 static void gst_cmml_dec_get_property (GObject * dec, guint property_id,
     GValue * value, GParamSpec * pspec);
 static void gst_cmml_dec_set_property (GObject * dec, guint property_id,
     const GValue * value, GParamSpec * pspec);
-static const GstQueryType *gst_cmml_dec_query_types (GstPad * pad);
-static gboolean gst_cmml_dec_sink_query (GstPad * pad, GstQuery * query);
-static gboolean gst_cmml_dec_sink_event (GstPad * pad, GstEvent * event);
+static gboolean gst_cmml_dec_sink_query (GstPad * pad, GstObject * parent,
+    GstQuery * query);
+static gboolean gst_cmml_dec_sink_event (GstPad * pad, GstObject * parent,
+    GstEvent * event);
 static gboolean gst_cmml_dec_convert (GstPad * pad, GstFormat src_fmt,
     gint64 src_val, GstFormat * dest_fmt, gint64 * dest_val);
 static GstStateChangeReturn gst_cmml_dec_change_state (GstElement * element,
     GstStateChange transition);
-static GstFlowReturn gst_cmml_dec_chain (GstPad * pad, GstBuffer * buffer);
+static GstFlowReturn gst_cmml_dec_chain (GstPad * pad, GstObject * parent,
+    GstBuffer * buffer);
 
 static GstCmmlPacketType gst_cmml_dec_parse_packet_type (GstCmmlDec * dec,
-    GstBuffer * buffer);
-static void gst_cmml_dec_parse_ident_header (GstCmmlDec * dec, GstBuffer * buf);
-static void gst_cmml_dec_parse_first_header (GstCmmlDec * dec, GstBuffer * buf);
-static void gst_cmml_dec_parse_preamble (GstCmmlDec * dec,
-    guchar * preamble, guchar * cmml_root_element);
-static void gst_cmml_dec_parse_xml (GstCmmlDec * dec,
-    guchar * data, guint size);
+    gchar * data, gsize size);
+static void gst_cmml_dec_parse_ident_header (GstCmmlDec * dec, guint8 * data,
+    gsize size);
+static void gst_cmml_dec_parse_first_header (GstCmmlDec * dec, guint8 * data,
+    gsize size);
+static void gst_cmml_dec_parse_preamble (GstCmmlDec * dec, guchar * preamble,
+    guchar * cmml_root_element);
+static void gst_cmml_dec_parse_xml (GstCmmlDec * dec, guchar * data,
+    guint size);
 static void gst_cmml_dec_parse_head (GstCmmlDec * dec, GstCmmlTagHead * head);
 static void gst_cmml_dec_parse_clip (GstCmmlDec * dec, GstCmmlTagClip * clip);
 
@@ -117,45 +123,39 @@ static void gst_cmml_dec_send_clip_tag (GstCmmlDec * dec,
 static void gst_cmml_dec_finalize (GObject * object);
 
 static void
-gst_cmml_dec_base_init (gpointer g_class)
+gst_cmml_dec_class_init (GstCmmlDecClass * klass)
 {
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_cmml_dec_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_cmml_dec_src_factory);
-  gst_element_class_set_details_simple (element_class, "CMML stream decoder",
-      "Codec/Decoder",
-      "Decodes CMML streams", "Alessandro Decina <alessandro@nnva.org>");
-}
+  object_class->set_property = gst_cmml_dec_set_property;
+  object_class->get_property = gst_cmml_dec_get_property;
+  object_class->finalize = gst_cmml_dec_finalize;
 
-static void
-gst_cmml_dec_class_init (GstCmmlDecClass * dec_class)
-{
-  GObjectClass *klass = G_OBJECT_CLASS (dec_class);
-
-  GST_ELEMENT_CLASS (klass)->change_state = gst_cmml_dec_change_state;
-
-  klass->set_property = gst_cmml_dec_set_property;
-  klass->get_property = gst_cmml_dec_get_property;
-  klass->finalize = gst_cmml_dec_finalize;
-
-  g_object_class_install_property (klass, GST_CMML_DEC_WAIT_CLIP_END,
+  g_object_class_install_property (object_class, GST_CMML_DEC_WAIT_CLIP_END,
       g_param_spec_boolean ("wait-clip-end-time",
           "Wait clip end time",
           "Send a tag for a clip when the clip ends, setting its end-time. "
           "Use when you need to know both clip's start-time and end-time.",
           FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  element_class->change_state = gst_cmml_dec_change_state;
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_cmml_dec_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_cmml_dec_src_factory));
+  gst_element_class_set_static_metadata (element_class, "CMML stream decoder",
+      "Codec/Decoder",
+      "Decodes CMML streams", "Alessandro Decina <alessandro@nnva.org>");
 }
 
 static void
-gst_cmml_dec_init (GstCmmlDec * dec, GstCmmlDecClass * klass)
+gst_cmml_dec_init (GstCmmlDec * dec)
 {
   dec->sinkpad =
       gst_pad_new_from_static_template (&gst_cmml_dec_sink_factory, "sink");
   gst_pad_set_chain_function (dec->sinkpad, gst_cmml_dec_chain);
-  gst_pad_set_query_type_function (dec->sinkpad, gst_cmml_dec_query_types);
   gst_pad_set_query_function (dec->sinkpad, gst_cmml_dec_sink_query);
   gst_pad_set_event_function (dec->sinkpad, gst_cmml_dec_sink_event);
   gst_element_add_pad (GST_ELEMENT (dec), dec->sinkpad);
@@ -240,7 +240,7 @@ gst_cmml_dec_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
-  res = parent_class->change_state (element, transition);
+  res = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
@@ -255,19 +255,8 @@ gst_cmml_dec_change_state (GstElement * element, GstStateChange transition)
   return res;
 }
 
-static const GstQueryType *
-gst_cmml_dec_query_types (GstPad * pad)
-{
-  static const GstQueryType query_types[] = {
-    GST_QUERY_CONVERT,
-    0
-  };
-
-  return query_types;
-}
-
 static gboolean
-gst_cmml_dec_sink_query (GstPad * pad, GstQuery * query)
+gst_cmml_dec_sink_query (GstPad * pad, GstObject * parent, GstQuery * query)
 {
   gboolean res = FALSE;
 
@@ -319,9 +308,9 @@ gst_cmml_dec_convert (GstPad * pad,
 }
 
 static gboolean
-gst_cmml_dec_sink_event (GstPad * pad, GstEvent * event)
+gst_cmml_dec_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
-  GstCmmlDec *dec = GST_CMML_DEC (GST_PAD_PARENT (pad));
+  GstCmmlDec *dec = GST_CMML_DEC (parent);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
@@ -361,16 +350,23 @@ gst_cmml_dec_sink_event (GstPad * pad, GstEvent * event)
       break;
   }
 
-  return gst_pad_event_default (pad, event);
+  return gst_pad_event_default (pad, parent, event);
 }
 
 static GstFlowReturn
-gst_cmml_dec_chain (GstPad * pad, GstBuffer * buffer)
+gst_cmml_dec_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 {
-  GstCmmlDec *dec = GST_CMML_DEC (GST_PAD_PARENT (pad));
+  GstCmmlDec *dec = GST_CMML_DEC (parent);
   GstCmmlPacketType packet;
+  GstMapInfo map;
+  guint8 *data;
+  gsize size;
 
-  if (GST_BUFFER_SIZE (buffer) == 0) {
+  gst_buffer_map (buffer, &map, GST_MAP_READ);
+  data = map.data;
+  size = map.size;
+
+  if (size == 0) {
     /* the EOS page could be empty */
     dec->flow_return = GST_FLOW_OK;
     goto done;
@@ -381,25 +377,24 @@ gst_cmml_dec_chain (GstPad * pad, GstBuffer * buffer)
       dec->granulerate_n, dec->granulerate_d, dec->granuleshift);
 
   /* identify the packet type */
-  packet = gst_cmml_dec_parse_packet_type (dec, buffer);
+  packet = gst_cmml_dec_parse_packet_type (dec, (gchar *) data, size);
 
   /* handle the packet. the handler will set dec->flow_return */
   switch (packet) {
     case GST_CMML_PACKET_IDENT_HEADER:
       if (dec->sent_root == FALSE)
         /* don't parse the ident again in case of seeking to the beginning */
-        gst_cmml_dec_parse_ident_header (dec, buffer);
+        gst_cmml_dec_parse_ident_header (dec, data, size);
       break;
     case GST_CMML_PACKET_FIRST_HEADER:
       if (dec->sent_root == FALSE)
         /* don't parse the xml preamble if it has already been parsed because it
          * would error out, so seeking to the beginning would fail */
-        gst_cmml_dec_parse_first_header (dec, buffer);
+        gst_cmml_dec_parse_first_header (dec, data, size);
       break;
     case GST_CMML_PACKET_SECOND_HEADER:
     case GST_CMML_PACKET_CLIP:
-      gst_cmml_dec_parse_xml (dec,
-          GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer));
+      gst_cmml_dec_parse_xml (dec, data, size);
       break;
     case GST_CMML_PACKET_UNKNOWN:
     default:
@@ -408,18 +403,18 @@ gst_cmml_dec_chain (GstPad * pad, GstBuffer * buffer)
   }
 
 done:
+  gst_buffer_unmap (buffer, &map);
   gst_buffer_unref (buffer);
+
   return dec->flow_return;
 }
 
 /* finds the packet type of the buffer
  */
 static GstCmmlPacketType
-gst_cmml_dec_parse_packet_type (GstCmmlDec * dec, GstBuffer * buffer)
+gst_cmml_dec_parse_packet_type (GstCmmlDec * dec, gchar * data, gsize size)
 {
   GstCmmlPacketType packet_type = GST_CMML_PACKET_UNKNOWN;
-  gchar *data = (gchar *) GST_BUFFER_DATA (buffer);
-  guint size = GST_BUFFER_SIZE (buffer);
 
   if (size >= 8 && !memcmp (data, "CMML\0\0\0\0", 8)) {
     packet_type = GST_CMML_PACKET_IDENT_HEADER;
@@ -443,19 +438,15 @@ gst_cmml_dec_new_buffer (GstCmmlDec * dec,
 {
   GstFlowReturn res;
 
-  res = gst_pad_alloc_buffer (dec->srcpad, GST_BUFFER_OFFSET_NONE,
-      size, gst_static_pad_template_get_caps (&gst_cmml_dec_src_factory),
-      buffer);
-
-  if (res == GST_FLOW_OK) {
+  *buffer = gst_buffer_new_allocate (NULL, size, NULL);
+  if (*buffer != NULL) {
     if (data)
-      memcpy (GST_BUFFER_DATA (*buffer), data, size);
+      gst_buffer_fill (*buffer, 0, data, size);
     GST_BUFFER_TIMESTAMP (*buffer) = dec->timestamp;
-  } else if (res == GST_FLOW_NOT_LINKED) {
-    GST_DEBUG_OBJECT (dec, "alloc function return NOT-LINKED, ignoring");
+    res = GST_FLOW_OK;
   } else {
-    GST_WARNING_OBJECT (dec, "alloc function returned error %s",
-        gst_flow_get_name (res));
+    GST_WARNING_OBJECT (dec, "could not allocate buffer");
+    res = GST_FLOW_ERROR;
   }
 
   return res;
@@ -464,14 +455,12 @@ gst_cmml_dec_new_buffer (GstCmmlDec * dec,
 /* parses the first CMML packet (the ident header)
  */
 static void
-gst_cmml_dec_parse_ident_header (GstCmmlDec * dec, GstBuffer * buffer)
+gst_cmml_dec_parse_ident_header (GstCmmlDec * dec, guint8 * data, gsize size)
 {
-  guint8 *data = GST_BUFFER_DATA (buffer);
-
   /* the ident header has a fixed length */
-  if (GST_BUFFER_SIZE (buffer) != CMML_IDENT_HEADER_SIZE) {
+  if (size != CMML_IDENT_HEADER_SIZE) {
     GST_ELEMENT_ERROR (dec, STREAM, DECODE,
-        (NULL), ("wrong ident header size: %d", GST_BUFFER_SIZE (buffer)));
+        (NULL), ("wrong ident header size: %" G_GSIZE_FORMAT, size));
     dec->flow_return = GST_FLOW_ERROR;
 
     return;
@@ -503,17 +492,15 @@ gst_cmml_dec_parse_ident_header (GstCmmlDec * dec, GstBuffer * buffer)
  * optional "cmml" processing instruction.
  */
 static void
-gst_cmml_dec_parse_first_header (GstCmmlDec * dec, GstBuffer * buffer)
+gst_cmml_dec_parse_first_header (GstCmmlDec * dec, guint8 * data, gsize size)
 {
-  gst_cmml_dec_parse_xml (dec,
-      GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer));
+  gst_cmml_dec_parse_xml (dec, data, size);
 
   /* if there is a processing instruction, gst_cmml_dec_parse_preamble
    * will be triggered. Otherwise we need to call it manually.
    */
   if (dec->flow_return == GST_FLOW_OK && !dec->sent_root) {
-    guchar *preamble = (guchar *) g_strndup ((gchar *) GST_BUFFER_DATA (buffer),
-        GST_BUFFER_SIZE (buffer));
+    guchar *preamble = (guchar *) g_strndup ((gchar *) data, size);
 
     gst_cmml_dec_parse_preamble (dec, preamble, (guchar *) "<cmml>");
     g_free (preamble);
@@ -579,17 +566,18 @@ gst_cmml_dec_parse_head (GstCmmlDec * dec, GstCmmlTagHead * head)
   GST_DEBUG_OBJECT (dec, "found CMML head (title: %s base: %s)",
       head->title, head->base);
 
+  /* FIXME: what's the point of all this GValue transform crack? */
   /* create the GST_TAG_TITLE tag */
   g_value_init (&str_val, G_TYPE_STRING);
   g_value_init (&title_val, gst_tag_get_type (GST_TAG_TITLE));
   g_value_set_string (&str_val, (gchar *) head->title);
   g_value_transform (&str_val, &title_val);
 
-  tags = gst_tag_list_new ();
+  tags = gst_tag_list_new_empty ();
   gst_tag_list_add_values (tags, GST_TAG_MERGE_APPEND,
       GST_TAG_TITLE, &title_val, NULL);
   gst_tag_list_add (tags, GST_TAG_MERGE_APPEND, GST_TAG_CMML_HEAD, head, NULL);
-  gst_element_found_tags_for_pad (GST_ELEMENT (dec), dec->srcpad, tags);
+  gst_pad_push_event (dec->srcpad, gst_event_new_tag (tags));
 
   g_value_unset (&str_val);
   g_value_unset (&title_val);
@@ -613,9 +601,8 @@ gst_cmml_dec_send_clip_tag (GstCmmlDec * dec, GstCmmlTagClip * clip)
 
   GST_DEBUG_OBJECT (dec, "sending clip tag %s", clip->id);
 
-  tags = gst_tag_list_new ();
-  gst_tag_list_add (tags, GST_TAG_MERGE_APPEND, GST_TAG_CMML_CLIP, clip, NULL);
-  gst_element_found_tags_for_pad (GST_ELEMENT (dec), dec->srcpad, tags);
+  tags = gst_tag_list_new (GST_TAG_CMML_CLIP, clip, NULL);
+  gst_pad_push_event (dec->srcpad, gst_event_new_tag (tags));
 }
 
 /* push the string representation of a clip */
