@@ -79,17 +79,21 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("text/x-cmml, encoded = (boolean) false")
     );
 
-GST_BOILERPLATE (GstCmmlEnc, gst_cmml_enc, GstElement, GST_TYPE_ELEMENT);
+#define gst_cmml_enc_parent_class parent_class
+G_DEFINE_TYPE (GstCmmlEnc, gst_cmml_enc, GST_TYPE_ELEMENT);
+
 static void gst_cmml_enc_get_property (GObject * object, guint property_id,
     GValue * value, GParamSpec * pspec);
 static void gst_cmml_enc_set_property (GObject * object, guint property_id,
     const GValue * value, GParamSpec * pspec);
-static gboolean gst_cmml_enc_sink_event (GstPad * pad, GstEvent * event);
+static gboolean gst_cmml_enc_sink_event (GstPad * pad, GstObject * parent,
+    GstEvent * event);
 static GstStateChangeReturn gst_cmml_enc_change_state (GstElement * element,
     GstStateChange transition);
-static GstFlowReturn gst_cmml_enc_chain (GstPad * pad, GstBuffer * buffer);
-static void gst_cmml_enc_parse_preamble (GstCmmlEnc * enc,
-    guchar * preamble, guchar * processing_instruction);
+static GstFlowReturn gst_cmml_enc_chain (GstPad * pad, GstObject * parent,
+    GstBuffer * buffer);
+static void gst_cmml_enc_parse_preamble (GstCmmlEnc * enc, guchar * preamble,
+    guchar * processing_instruction);
 static void gst_cmml_enc_parse_end_tag (GstCmmlEnc * enc);
 static void gst_cmml_enc_parse_tag_head (GstCmmlEnc * enc,
     GstCmmlTagHead * head);
@@ -103,24 +107,11 @@ static GstFlowReturn gst_cmml_enc_push (GstCmmlEnc * enc, GstBuffer * buffer);
 
 static void gst_cmml_enc_finalize (GObject * object);
 
-static void
-gst_cmml_enc_base_init (gpointer g_class)
-{
-  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_cmml_enc_sink_factory);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_cmml_enc_src_factory);
-  gst_element_class_set_details_simple (element_class, "CMML streams encoder",
-      "Codec/Encoder",
-      "Encodes CMML streams", "Alessandro Decina <alessandro@nnva.org>");
-}
-
 static void
 gst_cmml_enc_class_init (GstCmmlEncClass * enc_class)
 {
   GObjectClass *klass = G_OBJECT_CLASS (enc_class);
+  GstElementClass *element_class = GST_ELEMENT_CLASS (enc_class);
 
   klass->get_property = gst_cmml_enc_get_property;
   klass->set_property = gst_cmml_enc_set_property;
@@ -147,11 +138,19 @@ gst_cmml_enc_class_init (GstCmmlEncClass * enc_class)
           0, 64, 32,
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS));
 
-  GST_ELEMENT_CLASS (klass)->change_state = gst_cmml_enc_change_state;
+  element_class->change_state = gst_cmml_enc_change_state;
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_cmml_enc_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_cmml_enc_src_factory));
+  gst_element_class_set_static_metadata (element_class, "CMML streams encoder",
+      "Codec/Encoder",
+      "Encodes CMML streams", "Alessandro Decina <alessandro@nnva.org>");
 }
 
 static void
-gst_cmml_enc_init (GstCmmlEnc * enc, GstCmmlEncClass * klass)
+gst_cmml_enc_init (GstCmmlEnc * enc)
 {
   enc->sinkpad =
       gst_pad_new_from_static_template (&gst_cmml_enc_sink_factory, "sink");
@@ -250,7 +249,7 @@ gst_cmml_enc_change_state (GstElement * element, GstStateChange transition)
       break;
   }
 
-  res = parent_class->change_state (element, transition);
+  res = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
@@ -270,9 +269,9 @@ gst_cmml_enc_change_state (GstElement * element, GstStateChange transition)
 }
 
 static gboolean
-gst_cmml_enc_sink_event (GstPad * pad, GstEvent * event)
+gst_cmml_enc_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
-  GstCmmlEnc *enc = GST_CMML_ENC (GST_PAD_PARENT (pad));
+  GstCmmlEnc *enc = GST_CMML_ENC (parent);
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_EOS:
@@ -286,7 +285,7 @@ gst_cmml_enc_sink_event (GstPad * pad, GstEvent * event)
       break;
   }
 
-  return gst_pad_event_default (pad, event);
+  return gst_pad_event_default (pad, parent, event);
 }
 
 static GstFlowReturn
@@ -295,14 +294,14 @@ gst_cmml_enc_new_buffer (GstCmmlEnc * enc,
 {
   GstFlowReturn res;
 
-  res = gst_pad_alloc_buffer (enc->srcpad, GST_BUFFER_OFFSET_NONE, size,
-      NULL, buffer);
-  if (res == GST_FLOW_OK) {
+  *buffer = gst_buffer_new_allocate (NULL, size, NULL);
+  if (*buffer != NULL) {
     if (data)
-      memcpy (GST_BUFFER_DATA (*buffer), data, size);
+      gst_buffer_fill (*buffer, 0, data, size);
+    res = GST_FLOW_OK;
   } else {
-    GST_WARNING_OBJECT (enc, "alloc function returned error %s",
-        gst_flow_get_name (res));
+    GST_WARNING_OBJECT (enc, "could not allocate buffer");
+    res = GST_FLOW_ERROR;
   }
 
   return res;
@@ -339,9 +338,9 @@ gst_cmml_enc_set_header_on_caps (GstCmmlEnc * enc, GstCaps * caps,
   gst_value_array_append_value (&array, &value);
   gst_buffer_unref (buffer);
 
-  GST_BUFFER_FLAG_SET (ident, GST_BUFFER_FLAG_IN_CAPS);
-  GST_BUFFER_FLAG_SET (preamble, GST_BUFFER_FLAG_IN_CAPS);
-  GST_BUFFER_FLAG_SET (head, GST_BUFFER_FLAG_IN_CAPS);
+  GST_BUFFER_FLAG_SET (ident, GST_BUFFER_FLAG_HEADER);
+  GST_BUFFER_FLAG_SET (preamble, GST_BUFFER_FLAG_HEADER);
+  GST_BUFFER_FLAG_SET (head, GST_BUFFER_FLAG_HEADER);
 
   gst_structure_set_value (structure, "streamheader", &array);
 
@@ -443,15 +442,16 @@ gst_cmml_enc_parse_tag_head (GstCmmlEnc * enc, GstCmmlTagHead * head)
     goto alloc_error;
   headers = g_list_append (headers, head_buf);
 
-  caps = gst_pad_get_caps (enc->srcpad);
+  caps = gst_pad_query_caps (enc->srcpad, NULL);
   caps = gst_cmml_enc_set_header_on_caps (enc, caps,
       ident_buf, preamble_buf, head_buf);
+  gst_pad_set_caps (enc->srcpad, caps);
+  gst_caps_unref (caps);
 
   while (headers) {
     buffer = GST_BUFFER (headers->data);
     /* set granulepos 0 on headers */
     GST_BUFFER_OFFSET_END (buffer) = 0;
-    gst_buffer_set_caps (buffer, caps);
 
     enc->flow_return = gst_cmml_enc_push (enc, buffer);
     headers = g_list_delete_link (headers, headers);
@@ -460,8 +460,6 @@ gst_cmml_enc_parse_tag_head (GstCmmlEnc * enc, GstCmmlTagHead * head)
       goto push_error;
   }
 
-  gst_caps_unref (caps);
-
   enc->sent_headers = TRUE;
   return;
 
@@ -471,7 +469,6 @@ flow_unexpected:
   enc->flow_return = GST_FLOW_ERROR;
   return;
 push_error:
-  gst_caps_unref (caps);
   /* fallthrough */
 alloc_error:
   for (walk = headers; walk; walk = walk->next)
@@ -596,23 +593,27 @@ gst_cmml_enc_push (GstCmmlEnc * enc, GstBuffer * buffer)
 }
 
 static GstFlowReturn
-gst_cmml_enc_chain (GstPad * pad, GstBuffer * buffer)
+gst_cmml_enc_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
 {
   GError *err = NULL;
-  GstCmmlEnc *enc = GST_CMML_ENC (GST_PAD_PARENT (pad));
+  GstCmmlEnc *enc = GST_CMML_ENC (parent);
+  GstMapInfo map;
 
   /* the CMML handlers registered with enc->parser will override this when
    * encoding/pushing the buffers downstream
    */
   enc->flow_return = GST_FLOW_OK;
 
-  if (!gst_cmml_parser_parse_chunk (enc->parser,
-          (gchar *) GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer), &err)) {
+  gst_buffer_map (buffer, &map, GST_MAP_READ);
+
+  if (!gst_cmml_parser_parse_chunk (enc->parser, (gchar *) map.data, map.size,
+          &err)) {
     GST_ELEMENT_ERROR (enc, STREAM, ENCODE, (NULL), ("%s", err->message));
     g_error_free (err);
     enc->flow_return = GST_FLOW_ERROR;
   }
 
+  gst_buffer_unmap (buffer, &map);
   gst_buffer_unref (buffer);
   return enc->flow_return;
 }
