@@ -4,9 +4,6 @@
  * Filter:
  * Copyright (C) 2000 Donald A. Graft
  *
- * Copyright (C) 2012 Collabora Ltd.
- *	Author : Edward Hervey <edward@collabora.com>
- *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
@@ -30,8 +27,6 @@
 #include <string.h>
 #include <gst/gst.h>
 #include "gstpngenc.h"
-#include <gst/video/video.h>
-#include <gst/video/gstvideometa.h>
 #include <zlib.h>
 
 GST_DEBUG_CATEGORY_STATIC (pngenc_debug);
@@ -45,12 +40,14 @@ enum
 };
 
 #define DEFAULT_SNAPSHOT                FALSE
+/* #define DEFAULT_NEWMEDIA             FALSE */
 #define DEFAULT_COMPRESSION_LEVEL       6
 
 enum
 {
   ARG_0,
   ARG_SNAPSHOT,
+/*   ARG_NEWMEDIA, */
   ARG_COMPRESSION_LEVEL
 };
 
@@ -70,22 +67,19 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS (GST_VIDEO_CAPS_MAKE ("{ RGBA, RGB, GRAY8, GRAY16_BE }"))
     );
 
-#define parent_class gst_pngenc_parent_class
-G_DEFINE_TYPE (GstPngEnc, gst_pngenc, GST_TYPE_VIDEO_ENCODER);
+/* static GstElementClass *parent_class = NULL; */
+
+G_DEFINE_TYPE (GstPngEnc, gst_pngenc, GST_TYPE_ELEMENT);
 
 static void gst_pngenc_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec);
 static void gst_pngenc_get_property (GObject * object,
     guint prop_id, GValue * value, GParamSpec * pspec);
 
-static GstFlowReturn gst_pngenc_handle_frame (GstVideoEncoder * encoder,
-    GstVideoCodecFrame * frame);
-static gboolean gst_pngenc_set_format (GstVideoEncoder * encoder,
-    GstVideoCodecState * state);
-static gboolean gst_pngenc_propose_allocation (GstVideoEncoder * encoder,
-    GstQuery * query);
-
-static void gst_pngenc_finalize (GObject * object);
+static GstFlowReturn gst_pngenc_chain (GstPad * pad, GstObject * parent,
+    GstBuffer * data);
+static gboolean gst_pngenc_sink_event (GstPad * pad, GstObject * parent,
+    GstEvent * event);
 
 static void
 user_error_fn (png_structp png_ptr, png_const_charp error_msg)
@@ -104,13 +98,9 @@ gst_pngenc_class_init (GstPngEncClass * klass)
 {
   GObjectClass *gobject_class;
   GstElementClass *element_class;
-  GstVideoEncoderClass *venc_class;
 
   gobject_class = (GObjectClass *) klass;
   element_class = (GstElementClass *) klass;
-  venc_class = (GstVideoEncoderClass *) klass;
-
-  parent_class = g_type_class_peek_parent (klass);
 
   gobject_class->get_property = gst_pngenc_get_property;
   gobject_class->set_property = gst_pngenc_set_property;
@@ -120,6 +110,11 @@ gst_pngenc_class_init (GstPngEncClass * klass)
           "Send EOS after encoding a frame, useful for snapshots",
           DEFAULT_SNAPSHOT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+/*   g_object_class_install_property (gobject_class, ARG_NEWMEDIA, */
+/*       g_param_spec_boolean ("newmedia", "newmedia", */
+/*           "Send new media discontinuity after encoding each frame", */
+/*           DEFAULT_NEWMEDIA, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)); */
+
   g_object_class_install_property (gobject_class, ARG_COMPRESSION_LEVEL,
       g_param_spec_uint ("compression-level", "compression-level",
           "PNG compression level",
@@ -131,66 +126,78 @@ gst_pngenc_class_init (GstPngEncClass * klass)
       (element_class, gst_static_pad_template_get (&pngenc_sink_template));
   gst_element_class_add_pad_template
       (element_class, gst_static_pad_template_get (&pngenc_src_template));
-  gst_element_class_set_metadata (element_class, "PNG image encoder",
+  gst_element_class_set_static_metadata (element_class, "PNG image encoder",
       "Codec/Encoder/Image",
       "Encode a video frame to a .png image",
       "Jeremy SIMON <jsimon13@yahoo.fr>");
 
-  venc_class->set_format = gst_pngenc_set_format;
-  venc_class->handle_frame = gst_pngenc_handle_frame;
-  venc_class->propose_allocation = gst_pngenc_propose_allocation;
-  gobject_class->finalize = gst_pngenc_finalize;
-
   GST_DEBUG_CATEGORY_INIT (pngenc_debug, "pngenc", 0, "PNG image encoder");
 }
 
 
 static gboolean
-gst_pngenc_set_format (GstVideoEncoder * encoder, GstVideoCodecState * state)
+gst_pngenc_setcaps (GstPngEnc * pngenc, GstCaps * caps)
 {
-  GstPngEnc *pngenc;
-  gboolean ret = TRUE;
-  GstVideoInfo *info;
-  GstVideoCodecState *output_state;
+  int fps_n, fps_d;
+  GstCaps *pcaps;
+  gboolean ret;
+  GstVideoInfo info;
 
-  pngenc = GST_PNGENC (encoder);
-  info = &state->info;
+  ret = gst_video_info_from_caps (&info, caps);
 
-  switch (GST_VIDEO_INFO_FORMAT (info)) {
+  if (G_UNLIKELY (!ret))
+    goto done;
+
+  pngenc->info = info;
+
+  switch (GST_VIDEO_INFO_FORMAT (&info)) {
     case GST_VIDEO_FORMAT_RGBA:
       pngenc->png_color_type = PNG_COLOR_TYPE_RGBA;
+      pngenc->depth = 8;
       break;
     case GST_VIDEO_FORMAT_RGB:
       pngenc->png_color_type = PNG_COLOR_TYPE_RGB;
+      pngenc->depth = 8;
       break;
     case GST_VIDEO_FORMAT_GRAY8:
+      pngenc->png_color_type = PNG_COLOR_TYPE_GRAY;
+      pngenc->depth = 8;
+      break;
     case GST_VIDEO_FORMAT_GRAY16_BE:
       pngenc->png_color_type = PNG_COLOR_TYPE_GRAY;
+      pngenc->depth = 16;
       break;
     default:
       ret = FALSE;
       goto done;
   }
 
-  switch (GST_VIDEO_INFO_FORMAT (info)) {
-    case GST_VIDEO_FORMAT_GRAY16_BE:
-      pngenc->depth = 16;
-      break;
-    default:                   /* GST_VIDEO_FORMAT_RGB and GST_VIDEO_FORMAT_GRAY8 */
-      pngenc->depth = 8;
-      break;
+  pngenc->width = GST_VIDEO_INFO_WIDTH (&info);
+  pngenc->height = GST_VIDEO_INFO_HEIGHT (&info);
+  fps_n = GST_VIDEO_INFO_FPS_N (&info);
+  fps_d = GST_VIDEO_INFO_FPS_D (&info);
+
+  if (G_UNLIKELY (pngenc->width < 16 || pngenc->width > 1000000 ||
+          pngenc->height < 16 || pngenc->height > 1000000)) {
+    ret = FALSE;
+    goto done;
   }
 
-  if (pngenc->input_state)
-    gst_video_codec_state_unref (pngenc->input_state);
-  pngenc->input_state = gst_video_codec_state_ref (state);
+  pcaps = gst_caps_new_simple ("image/png",
+      "width", G_TYPE_INT, pngenc->width,
+      "height", G_TYPE_INT, pngenc->height,
+      "framerate", GST_TYPE_FRACTION, fps_n, fps_d, NULL);
 
-  output_state =
-      gst_video_encoder_set_output_state (encoder,
-      gst_caps_new_empty_simple ("image/png"), state);
-  gst_video_codec_state_unref (output_state);
+  ret = gst_pad_set_caps (pngenc->srcpad, pcaps);
 
+  gst_caps_unref (pcaps);
+
+  /* Fall-through. */
 done:
+  if (G_UNLIKELY (!ret)) {
+    pngenc->width = 0;
+    pngenc->height = 0;
+  }
 
   return ret;
 }
@@ -198,25 +205,30 @@ done:
 static void
 gst_pngenc_init (GstPngEnc * pngenc)
 {
+  /* sinkpad */
+  pngenc->sinkpad = gst_pad_new_from_static_template
+      (&pngenc_sink_template, "sink");
+  gst_pad_set_chain_function (pngenc->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_pngenc_chain));
+  gst_pad_set_event_function (pngenc->sinkpad,
+      GST_DEBUG_FUNCPTR (gst_pngenc_sink_event));
+  gst_element_add_pad (GST_ELEMENT (pngenc), pngenc->sinkpad);
+
+  /* srcpad */
+  pngenc->srcpad = gst_pad_new_from_static_template
+      (&pngenc_src_template, "src");
+  gst_pad_use_fixed_caps (pngenc->srcpad);
+  gst_element_add_pad (GST_ELEMENT (pngenc), pngenc->srcpad);
+
   /* init settings */
   pngenc->png_struct_ptr = NULL;
   pngenc->png_info_ptr = NULL;
 
   pngenc->snapshot = DEFAULT_SNAPSHOT;
+/*   pngenc->newmedia = FALSE; */
   pngenc->compression_level = DEFAULT_COMPRESSION_LEVEL;
 }
 
-static void
-gst_pngenc_finalize (GObject * object)
-{
-  GstPngEnc *pngenc = GST_PNGENC (object);
-
-  if (pngenc->input_state)
-    gst_video_codec_state_unref (pngenc->input_state);
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
-}
-
 static void
 user_flush_data (png_structp png_ptr G_GNUC_UNUSED)
 {
@@ -226,64 +238,80 @@ static void
 user_write_data (png_structp png_ptr, png_bytep data, png_uint_32 length)
 {
   GstPngEnc *pngenc;
-  GstMemory *mem;
-  GstMapInfo minfo;
+  GstMapInfo map;
 
   pngenc = (GstPngEnc *) png_get_io_ptr (png_ptr);
 
-  mem = gst_allocator_alloc (NULL, length, NULL);
-  if (!mem) {
-    GST_ERROR_OBJECT (pngenc, "Failed to allocate memory");
-    png_error (png_ptr, "Failed to allocate memory");
+  gst_buffer_map (pngenc->buffer_out, &map, GST_MAP_WRITE);
+  if (pngenc->written + length >= map.size) {
+    gst_buffer_unmap (pngenc->buffer_out, &map);
+    GST_ERROR_OBJECT (pngenc, "output buffer bigger than the input buffer!?");
+    png_error (png_ptr, "output buffer bigger than the input buffer!?");
 
     /* never reached */
     return;
   }
 
-  if (!gst_memory_map (mem, &minfo, GST_MAP_WRITE)) {
-    GST_ERROR_OBJECT (pngenc, "Failed to map memory");
-    gst_memory_unref (mem);
+  GST_DEBUG_OBJECT (pngenc, "writing %u bytes", (guint) length);
 
-    png_error (png_ptr, "Failed to map memory");
-
-    /* never reached */
-    return;
-  }
-
-  memcpy (minfo.data, data, length);
-  gst_memory_unmap (mem, &minfo);
-
-  gst_buffer_append_memory (pngenc->buffer_out, mem);
+  memcpy (map.data + pngenc->written, data, length);
+  gst_buffer_unmap (pngenc->buffer_out, &map);
+  pngenc->written += length;
 }
 
 static GstFlowReturn
-gst_pngenc_handle_frame (GstVideoEncoder * encoder, GstVideoCodecFrame * frame)
+gst_pngenc_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
 {
   GstPngEnc *pngenc;
   gint row_index;
   png_byte **row_pointers;
   GstFlowReturn ret = GST_FLOW_OK;
-  GstVideoInfo *info;
-  GstVideoFrame vframe;
+  GstBuffer *encoded_buf = NULL;
+  GstVideoFrame frame;
 
-  pngenc = GST_PNGENC (encoder);
-  info = &pngenc->input_state->info;
+  pngenc = GST_PNGENC (parent);
 
   GST_DEBUG_OBJECT (pngenc, "BEGINNING");
 
+  if (G_UNLIKELY (pngenc->width <= 0 || pngenc->height <= 0)) {
+    ret = GST_FLOW_NOT_NEGOTIATED;
+    goto exit;
+  }
+
+  if (!gst_video_frame_map (&frame, &pngenc->info, buf, GST_MAP_READ)) {
+    GST_ELEMENT_ERROR (pngenc, STREAM, FORMAT, (NULL),
+        ("Failed to map video frame, caps problem?"));
+    ret = GST_FLOW_ERROR;
+    goto exit;
+  }
+
   /* initialize png struct stuff */
   pngenc->png_struct_ptr = png_create_write_struct (PNG_LIBPNG_VER_STRING,
       (png_voidp) NULL, user_error_fn, user_warning_fn);
-  if (pngenc->png_struct_ptr == NULL)
-    goto struct_init_fail;
+  if (pngenc->png_struct_ptr == NULL) {
+    GST_ELEMENT_ERROR (pngenc, LIBRARY, INIT, (NULL),
+        ("Failed to initialize png structure"));
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
 
   pngenc->png_info_ptr = png_create_info_struct (pngenc->png_struct_ptr);
-  if (!pngenc->png_info_ptr)
-    goto png_info_fail;
+  if (!pngenc->png_info_ptr) {
+    png_destroy_write_struct (&(pngenc->png_struct_ptr), (png_infopp) NULL);
+    GST_ELEMENT_ERROR (pngenc, LIBRARY, INIT, (NULL),
+        ("Failed to initialize the png info structure"));
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
 
   /* non-0 return is from a longjmp inside of libpng */
-  if (setjmp (png_jmpbuf (pngenc->png_struct_ptr)) != 0)
-    goto longjmp_fail;
+  if (setjmp (png_jmpbuf (pngenc->png_struct_ptr)) != 0) {
+    png_destroy_write_struct (&pngenc->png_struct_ptr, &pngenc->png_info_ptr);
+    GST_ELEMENT_ERROR (pngenc, LIBRARY, FAILED, (NULL),
+        ("returning from longjmp"));
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
 
   png_set_filter (pngenc->png_struct_ptr, 0,
       PNG_FILTER_NONE | PNG_FILTER_VALUE_NONE);
@@ -291,8 +319,8 @@ gst_pngenc_handle_frame (GstVideoEncoder * encoder, GstVideoCodecFrame * frame)
 
   png_set_IHDR (pngenc->png_struct_ptr,
       pngenc->png_info_ptr,
-      GST_VIDEO_INFO_WIDTH (info),
-      GST_VIDEO_INFO_HEIGHT (info),
+      pngenc->width,
+      pngenc->height,
       pngenc->depth,
       pngenc->png_color_type,
       PNG_INTERLACE_NONE,
@@ -301,22 +329,17 @@ gst_pngenc_handle_frame (GstVideoEncoder * encoder, GstVideoCodecFrame * frame)
   png_set_write_fn (pngenc->png_struct_ptr, pngenc,
       (png_rw_ptr) user_write_data, user_flush_data);
 
-  row_pointers = g_new (png_byte *, GST_VIDEO_INFO_HEIGHT (info));
-  if (!gst_video_frame_map (&vframe, &pngenc->input_state->info,
-          frame->input_buffer, GST_MAP_READ)) {
-    GST_ELEMENT_ERROR (pngenc, STREAM, FORMAT, (NULL),
-        ("Failed to map video frame, caps problem?"));
-    ret = GST_FLOW_ERROR;
-    goto done;
-  }
+  row_pointers = g_new (png_byte *, pngenc->height);
 
-  for (row_index = 0; row_index < GST_VIDEO_INFO_HEIGHT (info); row_index++) {
-    row_pointers[row_index] = GST_VIDEO_FRAME_COMP_DATA (&vframe, 0) +
-        (row_index * GST_VIDEO_FRAME_COMP_STRIDE (&vframe, 0));
+  for (row_index = 0; row_index < pngenc->height; row_index++) {
+    row_pointers[row_index] = GST_VIDEO_FRAME_COMP_DATA (&frame, 0) +
+        (row_index * GST_VIDEO_FRAME_COMP_STRIDE (&frame, 0));
   }
 
   /* allocate the output buffer */
-  pngenc->buffer_out = gst_buffer_new ();
+  pngenc->buffer_out =
+      gst_buffer_new_and_alloc (pngenc->height * pngenc->width);
+  pngenc->written = 0;
 
   png_write_info (pngenc->png_struct_ptr, pngenc->png_info_ptr);
   png_write_image (pngenc->png_struct_ptr, row_pointers);
@@ -324,57 +347,69 @@ gst_pngenc_handle_frame (GstVideoEncoder * encoder, GstVideoCodecFrame * frame)
 
   g_free (row_pointers);
 
+  GST_DEBUG_OBJECT (pngenc, "written %d", pngenc->written);
+
+  encoded_buf =
+      gst_buffer_copy_region (pngenc->buffer_out, GST_BUFFER_COPY_MEMORY,
+      0, pngenc->written);
+
   png_destroy_info_struct (pngenc->png_struct_ptr, &pngenc->png_info_ptr);
   png_destroy_write_struct (&pngenc->png_struct_ptr, (png_infopp) NULL);
 
-  /* Set final size and store */
-  frame->output_buffer = pngenc->buffer_out;
+  GST_BUFFER_TIMESTAMP (encoded_buf) = GST_BUFFER_TIMESTAMP (buf);
+  GST_BUFFER_DURATION (encoded_buf) = GST_BUFFER_DURATION (buf);
 
-  pngenc->buffer_out = NULL;
-
-  if ((ret = gst_video_encoder_finish_frame (encoder, frame)) != GST_FLOW_OK)
+  if ((ret = gst_pad_push (pngenc->srcpad, encoded_buf)) != GST_FLOW_OK)
     goto done;
 
-  if (pngenc->snapshot)
+  if (pngenc->snapshot) {
+    GstEvent *event;
+
+    GST_DEBUG_OBJECT (pngenc, "snapshot mode, sending EOS");
+    /* send EOS event, since a frame has been pushed out */
+    event = gst_event_new_eos ();
+
+    gst_pad_push_event (pngenc->srcpad, event);
     ret = GST_FLOW_EOS;
+  }
 
 done:
+  gst_video_frame_unmap (&frame);
+exit:
+  gst_buffer_unref (buf);
   GST_DEBUG_OBJECT (pngenc, "END, ret:%d", ret);
 
+  if (pngenc->buffer_out != NULL) {
+    gst_buffer_unref (pngenc->buffer_out);
+    pngenc->buffer_out = NULL;
+  }
+
   return ret;
-
-  /* ERRORS */
-struct_init_fail:
-  {
-    GST_ELEMENT_ERROR (pngenc, LIBRARY, INIT, (NULL),
-        ("Failed to initialize png structure"));
-    return GST_FLOW_ERROR;
-  }
-
-png_info_fail:
-  {
-    png_destroy_write_struct (&(pngenc->png_struct_ptr), (png_infopp) NULL);
-    GST_ELEMENT_ERROR (pngenc, LIBRARY, INIT, (NULL),
-        ("Failed to initialize the png info structure"));
-    return GST_FLOW_ERROR;
-  }
-
-longjmp_fail:
-  {
-    png_destroy_write_struct (&pngenc->png_struct_ptr, &pngenc->png_info_ptr);
-    GST_ELEMENT_ERROR (pngenc, LIBRARY, FAILED, (NULL),
-        ("returning from longjmp"));
-    return GST_FLOW_ERROR;
-  }
 }
 
 static gboolean
-gst_pngenc_propose_allocation (GstVideoEncoder * encoder, GstQuery * query)
+gst_pngenc_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
-  gst_query_add_allocation_meta (query, GST_VIDEO_META_API_TYPE, NULL);
+  GstPngEnc *enc;
+  gboolean res;
 
-  return GST_VIDEO_ENCODER_CLASS (parent_class)->propose_allocation (encoder,
-      query);
+  enc = GST_PNGENC (parent);
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_CAPS:
+    {
+      GstCaps *caps;
+
+      gst_event_parse_caps (event, &caps);
+      res = gst_pngenc_setcaps (enc, caps);
+      gst_event_unref (event);
+      break;
+    }
+    default:
+      res = gst_pad_push_event (enc->srcpad, event);
+      break;
+  }
+  return res;
 }
 
 static void
@@ -389,6 +424,9 @@ gst_pngenc_get_property (GObject * object,
     case ARG_SNAPSHOT:
       g_value_set_boolean (value, pngenc->snapshot);
       break;
+/*     case ARG_NEWMEDIA: */
+/*       g_value_set_boolean (value, pngenc->newmedia); */
+/*       break; */
     case ARG_COMPRESSION_LEVEL:
       g_value_set_uint (value, pngenc->compression_level);
       break;
@@ -411,6 +449,9 @@ gst_pngenc_set_property (GObject * object,
     case ARG_SNAPSHOT:
       pngenc->snapshot = g_value_get_boolean (value);
       break;
+/*     case ARG_NEWMEDIA: */
+/*       pngenc->newmedia = g_value_get_boolean (value); */
+/*       break; */
     case ARG_COMPRESSION_LEVEL:
       pngenc->compression_level = g_value_get_uint (value);
       break;
