@@ -29,7 +29,7 @@
  * <refsect2>
  * <title>Example pipelines</title>
  * |[
- * gst-launch-1.0 -v pulsesrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=alsasrc.ogg
+ * gst-launch -v pulsesrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=alsasrc.ogg
  * ]| Record from a sound card using pulseaudio and encode to Ogg/Vorbis.
  * </refsect2>
  */
@@ -94,19 +94,16 @@ static gboolean gst_pulsesrc_prepare (GstAudioSrc * asrc,
 static gboolean gst_pulsesrc_unprepare (GstAudioSrc * asrc);
 
 static guint gst_pulsesrc_read (GstAudioSrc * asrc, gpointer data,
-    guint length, GstClockTime * timestamp);
+    guint length);
 static guint gst_pulsesrc_delay (GstAudioSrc * asrc);
 
 static void gst_pulsesrc_reset (GstAudioSrc * src);
 
 static gboolean gst_pulsesrc_negotiate (GstBaseSrc * basesrc);
-static gboolean gst_pulsesrc_event (GstBaseSrc * basesrc, GstEvent * event);
 
 static GstStateChangeReturn gst_pulsesrc_change_state (GstElement *
     element, GstStateChange transition);
 
-static GstClockTime gst_pulsesrc_get_time (GstClock * clock, GstPulseSrc * src);
-
 #if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
 # define FORMATS "{ S16LE, S16BE, F32LE, F32BE, S32LE, S32BE, U8 }"
 #else
@@ -149,7 +146,6 @@ gst_pulsesrc_class_init (GstPulseSrcClass * klass)
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_pulsesrc_change_state);
 
-  gstbasesrc_class->event = GST_DEBUG_FUNCPTR (gst_pulsesrc_event);
   gstbasesrc_class->negotiate = GST_DEBUG_FUNCPTR (gst_pulsesrc_negotiate);
 
   gstaudiosrc_class->open = GST_DEBUG_FUNCPTR (gst_pulsesrc_open);
@@ -295,14 +291,6 @@ gst_pulsesrc_init (GstPulseSrc * pulsesrc)
   /* this should be the default but it isn't yet */
   gst_audio_base_src_set_slave_method (GST_AUDIO_BASE_SRC (pulsesrc),
       GST_AUDIO_BASE_SRC_SLAVE_SKEW);
-
-  /* override with a custom clock */
-  if (GST_AUDIO_BASE_SRC (pulsesrc)->clock)
-    gst_object_unref (GST_AUDIO_BASE_SRC (pulsesrc)->clock);
-
-  GST_AUDIO_BASE_SRC (pulsesrc)->clock =
-      gst_audio_clock_new ("GstPulseSrcClock",
-      (GstAudioClockGetTimeFunc) gst_pulsesrc_get_time, pulsesrc, NULL);
 }
 
 static void
@@ -368,9 +356,6 @@ gst_pulsesrc_finalize (GObject * object)
 static gboolean
 gst_pulsesrc_is_dead (GstPulseSrc * pulsesrc, gboolean check_stream)
 {
-  if (!pulsesrc->stream_connected)
-    return TRUE;
-
   if (!CONTEXT_OK (pulsesrc->context))
     goto error;
 
@@ -995,23 +980,19 @@ gst_pulsesrc_unprepare (GstAudioSrc * asrc)
 }
 
 static guint
-gst_pulsesrc_read (GstAudioSrc * asrc, gpointer data, guint length,
-    GstClockTime * timestamp)
+gst_pulsesrc_read (GstAudioSrc * asrc, gpointer data, guint length)
 {
   GstPulseSrc *pulsesrc = GST_PULSESRC_CAST (asrc);
   size_t sum = 0;
 
+  pa_threaded_mainloop_lock (pulsesrc->mainloop);
+  pulsesrc->in_read = TRUE;
+
   if (g_atomic_int_compare_and_exchange (&pulsesrc->notify, 1, 0)) {
     g_object_notify (G_OBJECT (pulsesrc), "volume");
     g_object_notify (G_OBJECT (pulsesrc), "mute");
   }
 
-  pa_threaded_mainloop_lock (pulsesrc->mainloop);
-  pulsesrc->in_read = TRUE;
-
-  if (!pulsesrc->stream_connected)
-    goto not_connected;
-
   if (pulsesrc->paused)
     goto was_paused;
 
@@ -1077,11 +1058,6 @@ gst_pulsesrc_read (GstAudioSrc * asrc, gpointer data, guint length,
   return sum;
 
   /* ERRORS */
-not_connected:
-  {
-    GST_LOG_OBJECT (pulsesrc, "we are not connected");
-    goto unlock_and_fail;
-  }
 was_paused:
   {
     GST_LOG_OBJECT (pulsesrc, "we are paused");
@@ -1150,71 +1126,49 @@ server_dead:
 }
 
 static gboolean
-gst_pulsesrc_create_stream (GstPulseSrc * pulsesrc, GstCaps ** caps,
-    GstAudioRingBufferSpec * rspec)
+gst_pulsesrc_create_stream (GstPulseSrc * pulsesrc, GstCaps ** caps)
 {
   pa_channel_map channel_map;
   const pa_channel_map *m;
   GstStructure *s;
   gboolean need_channel_layout = FALSE;
-  GstAudioRingBufferSpec new_spec, *spec;
+  GstAudioRingBufferSpec spec;
   const gchar *name;
-  int i;
 
-  /* If we already have a stream (renegotiation), free it first */
-  if (pulsesrc->stream)
-    gst_pulsesrc_destroy_stream (pulsesrc);
-
-  if (rspec) {
-    /* Post-negotiation, we already have a ringbuffer spec, so we just need to
-     * use it to create a stream. */
-    spec = rspec;
-
-    /* At this point, we expect the channel-mask to be set in caps, so we just
-     * use that */
-    if (!gst_pulse_gst_to_channel_map (&channel_map, spec))
-      goto invalid_spec;
-
-  } else if (caps) {
-    /* At negotiation time, we get a fixed caps and use it to set up a stream */
-    s = gst_caps_get_structure (*caps, 0);
-    gst_structure_get_int (s, "channels", &new_spec.info.channels);
-    if (!gst_structure_has_field (s, "channel-mask")) {
-      if (new_spec.info.channels == 1) {
-        pa_channel_map_init_mono (&channel_map);
-      } else if (new_spec.info.channels == 2) {
-        pa_channel_map_init_stereo (&channel_map);
-      } else {
-        need_channel_layout = TRUE;
-        gst_structure_set (s, "channel-mask", GST_TYPE_BITMASK,
-            G_GUINT64_CONSTANT (0), NULL);
-      }
-    }
-
-    memset (&new_spec, 0, sizeof (GstAudioRingBufferSpec));
-    new_spec.latency_time = GST_SECOND;
-    if (!gst_audio_ring_buffer_parse_caps (&new_spec, *caps))
-      goto invalid_caps;
-
-    /* Keep the refcount of the caps at 1 to make them writable */
-    gst_caps_unref (new_spec.caps);
-
-    if (!need_channel_layout
-        && !gst_pulse_gst_to_channel_map (&channel_map, &new_spec)) {
+  s = gst_caps_get_structure (*caps, 0);
+  gst_structure_get_int (s, "channels", &spec.info.channels);
+  if (!gst_structure_has_field (s, "channel-mask")) {
+    if (spec.info.channels == 1) {
+      pa_channel_map_init_mono (&channel_map);
+    } else if (spec.info.channels == 2) {
+      gst_structure_set (s, "channel-mask", GST_TYPE_BITMASK,
+          GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_LEFT) |
+          GST_AUDIO_CHANNEL_POSITION_MASK (FRONT_RIGHT), NULL);
+      pa_channel_map_init_stereo (&channel_map);
+    } else {
       need_channel_layout = TRUE;
       gst_structure_set (s, "channel-mask", GST_TYPE_BITMASK,
           G_GUINT64_CONSTANT (0), NULL);
-      for (i = 0; i < G_N_ELEMENTS (new_spec.info.position); i++)
-        new_spec.info.position[i] = GST_AUDIO_CHANNEL_POSITION_INVALID;
     }
-
-    spec = &new_spec;
-  } else {
-    /* !rspec && !caps */
-    g_assert_not_reached ();
   }
 
-  if (!gst_pulse_fill_sample_spec (spec, &pulsesrc->sample_spec))
+  memset (&spec, 0, sizeof (GstAudioRingBufferSpec));
+  spec.latency_time = GST_SECOND;
+  if (!gst_audio_ring_buffer_parse_caps (&spec, *caps))
+    goto invalid_caps;
+
+  /* Keep the refcount of the caps at 1 to make them writable */
+  gst_caps_unref (spec.caps);
+
+  if (!need_channel_layout
+      && !gst_pulse_gst_to_channel_map (&channel_map, &spec)) {
+    need_channel_layout = TRUE;
+    gst_structure_set (s, "channel-mask", GST_TYPE_BITMASK,
+        G_GUINT64_CONSTANT (0), NULL);
+    memset (spec.info.position, 0xff, sizeof (spec.info.position));
+  }
+
+  if (!gst_pulse_fill_sample_spec (&spec, &pulsesrc->sample_spec))
     goto invalid_spec;
 
   pa_threaded_mainloop_lock (pulsesrc->mainloop);
@@ -1235,17 +1189,14 @@ gst_pulsesrc_create_stream (GstPulseSrc * pulsesrc, GstCaps ** caps,
               (need_channel_layout) ? NULL : &channel_map)))
     goto create_failed;
 
-  if (caps) {
-    m = pa_stream_get_channel_map (pulsesrc->stream);
-    gst_pulse_channel_map_to_gst (m, &new_spec);
-    gst_audio_channel_positions_to_valid_order (new_spec.info.position,
-        new_spec.info.channels);
-    gst_caps_unref (*caps);
-    *caps = gst_audio_info_to_caps (&new_spec.info);
-
-    GST_DEBUG_OBJECT (pulsesrc, "Caps are %" GST_PTR_FORMAT, *caps);
-  }
+  m = pa_stream_get_channel_map (pulsesrc->stream);
+  gst_pulse_channel_map_to_gst (m, &spec);
+  gst_audio_channel_positions_to_valid_order (spec.info.position,
+      spec.info.channels);
+  gst_caps_unref (*caps);
+  *caps = gst_audio_info_to_caps (&spec.info);
 
+  GST_DEBUG_OBJECT (pulsesrc, "Caps are %" GST_PTR_FORMAT, *caps);
 
   pa_stream_set_state_callback (pulsesrc->stream, gst_pulsesrc_stream_state_cb,
       pulsesrc);
@@ -1298,21 +1249,6 @@ unlock_and_fail:
   }
 }
 
-static gboolean
-gst_pulsesrc_event (GstBaseSrc * basesrc, GstEvent * event)
-{
-  GST_DEBUG_OBJECT (basesrc, "handle event %" GST_PTR_FORMAT, event);
-
-  switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_RECONFIGURE:
-      gst_pad_check_reconfigure (GST_BASE_SRC_PAD (basesrc));
-      break;
-    default:
-      break;
-  }
-  return GST_BASE_SRC_CLASS (parent_class)->event (basesrc, event);
-}
-
 /* This is essentially gst_base_src_negotiate_default() but the caps
  * are guaranteed to have a channel layout for > 2 channels
  */
@@ -1360,7 +1296,7 @@ gst_pulsesrc_negotiate (GstBaseSrc * basesrc)
         result = TRUE;
       } else if (gst_caps_is_fixed (caps)) {
         /* yay, fixed caps, use those then */
-        result = gst_pulsesrc_create_stream (pulsesrc, &caps, NULL);
+        result = gst_pulsesrc_create_stream (pulsesrc, &caps);
         if (result)
           result = gst_base_src_set_caps (basesrc, caps);
       }
@@ -1386,13 +1322,9 @@ gst_pulsesrc_prepare (GstAudioSrc * asrc, GstAudioRingBufferSpec * spec)
   GstPulseSrc *pulsesrc = GST_PULSESRC_CAST (asrc);
   pa_stream_flags_t flags;
   pa_operation *o;
-  GstAudioClock *clock;
 
   pa_threaded_mainloop_lock (pulsesrc->mainloop);
 
-  if (!pulsesrc->stream)
-    gst_pulsesrc_create_stream (pulsesrc, NULL, spec);
-
   {
     GstAudioRingBufferSpec s = *spec;
     const pa_channel_map *m;
@@ -1406,7 +1338,7 @@ gst_pulsesrc_prepare (GstAudioSrc * asrc, GstAudioRingBufferSpec * spec)
   /* enable event notifications */
   GST_LOG_OBJECT (pulsesrc, "subscribing to context events");
   if (!(o = pa_context_subscribe (pulsesrc->context,
-              PA_SUBSCRIPTION_MASK_SOURCE_OUTPUT, NULL, NULL))) {
+              PA_SUBSCRIPTION_MASK_SINK_INPUT, NULL, NULL))) {
     GST_ELEMENT_ERROR (pulsesrc, RESOURCE, FAILED,
         ("pa_context_subscribe() failed: %s",
             pa_strerror (pa_context_errno (pulsesrc->context))), (NULL));
@@ -1439,10 +1371,6 @@ gst_pulsesrc_prepare (GstAudioSrc * asrc, GstAudioRingBufferSpec * spec)
     goto connect_failed;
   }
 
-  /* our clock will now start from 0 again */
-  clock = GST_AUDIO_CLOCK (GST_AUDIO_BASE_SRC (pulsesrc)->clock);
-  gst_audio_clock_reset (clock, 0);
-
   pulsesrc->corked = TRUE;
 
   for (;;) {
@@ -1678,11 +1606,6 @@ gst_pulsesrc_change_state (GstElement * element, GstStateChange transition)
         goto mainloop_start_failed;
       }
       break;
-    case GST_STATE_CHANGE_READY_TO_PAUSED:
-      gst_element_post_message (element,
-          gst_message_new_clock_provide (GST_OBJECT_CAST (element),
-              GST_AUDIO_BASE_SRC (this)->clock, TRUE));
-      break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       /* uncork and start recording */
       gst_pulsesrc_play (this);
@@ -1716,12 +1639,6 @@ gst_pulsesrc_change_state (GstElement * element, GstStateChange transition)
         this->mainloop = NULL;
       }
       break;
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
-      /* format_lost is reset in release() in baseaudiosink */
-      gst_element_post_message (element,
-          gst_message_new_clock_lost (GST_OBJECT_CAST (element),
-              GST_AUDIO_BASE_SRC (this)->clock));
-      break;
     default:
       break;
   }
@@ -1742,29 +1659,3 @@ mainloop_start_failed:
     return GST_STATE_CHANGE_FAILURE;
   }
 }
-
-static GstClockTime
-gst_pulsesrc_get_time (GstClock * clock, GstPulseSrc * src)
-{
-  pa_usec_t time = 0;
-
-  pa_threaded_mainloop_lock (src->mainloop);
-  if (!src->stream)
-    goto unlock_and_out;
-
-  if (gst_pulsesrc_is_dead (src, TRUE))
-    goto unlock_and_out;
-
-  if (pa_stream_get_time (src->stream, &time) < 0) {
-    GST_DEBUG_OBJECT (src, "could not get time");
-    time = GST_CLOCK_TIME_NONE;
-  } else {
-    time *= 1000;
-  }
-
-
-unlock_and_out:
-  pa_threaded_mainloop_unlock (src->mainloop);
-
-  return time;
-}
