@@ -40,7 +40,7 @@
  * <refsect2>
  * <title>Example pipelines</title>
  * |[
- * gst-launch-1.0 -v filesrc location=sine.flac ! flacparse ! identity \
+ * gst-launch -v filesrc location=sine.flac ! flacparse ! identity \
  *            ! oggmux ! filesink location=sine-remuxed.ogg
  * ]| This pipeline converts a native FLAC format file to an ogg bitstream.
  * It also illustrates that the streamheader is set in the caps, and that each
@@ -184,9 +184,6 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("audio/x-flac")
     );
 
-static GstBuffer *gst_flac_parse_generate_vorbiscomment (GstFlacParse *
-    flacparse);
-
 static void gst_flac_parse_finalize (GObject * object);
 static void gst_flac_parse_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -295,13 +292,9 @@ gst_flac_parse_finalize (GObject * object)
   GstFlacParse *flacparse = GST_FLAC_PARSE (object);
 
   if (flacparse->tags) {
-    gst_tag_list_unref (flacparse->tags);
+    gst_tag_list_free (flacparse->tags);
     flacparse->tags = NULL;
   }
-  if (flacparse->toc) {
-    gst_toc_unref (flacparse->toc);
-    flacparse->toc = NULL;
-  }
 
   g_list_foreach (flacparse->headers, (GFunc) gst_mini_object_unref, NULL);
   g_list_free (flacparse->headers);
@@ -350,13 +343,9 @@ gst_flac_parse_stop (GstBaseParse * parse)
   GstFlacParse *flacparse = GST_FLAC_PARSE (parse);
 
   if (flacparse->tags) {
-    gst_tag_list_unref (flacparse->tags);
+    gst_tag_list_free (flacparse->tags);
     flacparse->tags = NULL;
   }
-  if (flacparse->toc) {
-    gst_toc_unref (flacparse->toc);
-    flacparse->toc = NULL;
-  }
 
   g_list_foreach (flacparse->headers, (GFunc) gst_mini_object_unref, NULL);
   g_list_free (flacparse->headers);
@@ -560,16 +549,6 @@ gst_flac_parse_frame_header_is_valid (GstFlacParse * flacparse,
     }
   }
 
-  /* documentation says:
-   * The "blocking strategy" bit must be the same throughout the entire stream. */
-  if (flacparse->blocking_strategy != blocking_strategy) {
-    if (flacparse->block_size != 0) {
-      GST_WARNING_OBJECT (flacparse, "blocking strategy is not constant");
-      if (suspect)
-        *suspect = TRUE;
-    }
-  }
-
   /* 
      The FLAC format documentation says:
      The "blocking strategy" bit determines how to calculate the sample number
@@ -681,7 +660,7 @@ gst_flac_parse_frame_is_valid (GstFlacParse * flacparse,
           gst_flac_parse_frame_header_is_valid (flacparse, map.data + i,
           remaining, FALSE, NULL, &suspect_end);
       if (header_ret == FRAME_HEADER_VALID) {
-        if (flacparse->check_frame_checksums || suspect_start || suspect_end) {
+        if (flacparse->check_frame_checksums) {
           guint16 actual_crc = gst_flac_calculate_crc16 (map.data, i - 2);
           guint16 expected_crc = GST_READ_UINT16_BE (map.data + i - 2);
 
@@ -973,121 +952,13 @@ gst_flac_parse_handle_vorbiscomment (GstFlacParse * flacparse,
   if (flacparse->tags == NULL) {
     GST_ERROR_OBJECT (flacparse, "Invalid vorbiscomment block");
   } else if (gst_tag_list_is_empty (flacparse->tags)) {
-    gst_tag_list_unref (flacparse->tags);
+    gst_tag_list_free (flacparse->tags);
     flacparse->tags = NULL;
   }
 
   return TRUE;
 }
 
-static gboolean
-gst_flac_parse_handle_cuesheet (GstFlacParse * flacparse, GstBuffer * buffer)
-{
-  GstByteReader reader;
-  GstMapInfo map;
-  guint i, j;
-  guint8 n_tracks, track_num, index;
-  guint64 offset;
-  gint64 start, stop;
-  gchar *id;
-  gchar isrc[13];
-  GstTagList *tags;
-  GstToc *toc;
-  GstTocEntry *cur_entry = NULL, *prev_entry = NULL;
-
-  gst_buffer_map (buffer, &map, GST_MAP_READ);
-  gst_byte_reader_init (&reader, map.data, map.size);
-
-  toc = gst_toc_new (GST_TOC_SCOPE_GLOBAL);
-
-  /* skip 4 bytes METADATA_BLOCK_HEADER */
-  /* http://flac.sourceforge.net/format.html#metadata_block_header */
-  if (!gst_byte_reader_skip (&reader, 4))
-    goto error;
-
-  /* skip 395 bytes from METADATA_BLOCK_CUESHEET */
-  /* http://flac.sourceforge.net/format.html#metadata_block_cuesheet */
-  if (!gst_byte_reader_skip (&reader, 395))
-    goto error;
-
-  if (!gst_byte_reader_get_uint8 (&reader, &n_tracks))
-    goto error;
-
-  /* CUESHEET_TRACK */
-  /* http://flac.sourceforge.net/format.html#cuesheet_track */
-  for (i = 0; i < n_tracks; i++) {
-    if (!gst_byte_reader_get_uint64_be (&reader, &offset))
-      goto error;
-    if (!gst_byte_reader_get_uint8 (&reader, &track_num))
-      goto error;
-
-    if (gst_byte_reader_get_remaining (&reader) < 12)
-      goto error;
-    memcpy (isrc, map.data + gst_byte_reader_get_pos (&reader), 12);
-    /* \0-terminate the string */
-    isrc[12] = '\0';
-    if (!gst_byte_reader_skip (&reader, 12))
-      goto error;
-
-    /* skip 14 bytes from CUESHEET_TRACK */
-    if (!gst_byte_reader_skip (&reader, 14))
-      goto error;
-    if (!gst_byte_reader_get_uint8 (&reader, &index))
-      goto error;
-    /* add tracks in TOC */
-    /* lead-out tack has number 170 or 255 */
-    if (track_num != 170 && track_num != 255) {
-      prev_entry = cur_entry;
-      /* previous track stop time = current track start time */
-      if (prev_entry != NULL) {
-        gst_toc_entry_get_start_stop_times (prev_entry, &start, NULL);
-        stop =
-            gst_util_uint64_scale_round (offset, GST_SECOND,
-            flacparse->samplerate);
-        gst_toc_entry_set_start_stop_times (prev_entry, start, stop);
-      }
-      id = g_strdup_printf ("%08x", track_num);
-      cur_entry = gst_toc_entry_new (GST_TOC_ENTRY_TYPE_TRACK, id);
-      g_free (id);
-      start =
-          gst_util_uint64_scale_round (offset, GST_SECOND,
-          flacparse->samplerate);
-      gst_toc_entry_set_start_stop_times (cur_entry, start, -1);
-      /* add ISRC as tag in track */
-      if (strlen (isrc) != 0) {
-        tags = gst_tag_list_new_empty ();
-        gst_tag_list_add (tags, GST_TAG_MERGE_APPEND, GST_TAG_ISRC, isrc, NULL);
-        gst_toc_entry_set_tags (cur_entry, tags);
-      }
-      gst_toc_append_entry (toc, cur_entry);
-      /* CUESHEET_TRACK_INDEX */
-      /* http://flac.sourceforge.net/format.html#cuesheet_track_index */
-      for (j = 0; j < index; j++) {
-        if (!gst_byte_reader_skip (&reader, 12))
-          goto error;
-      }
-    } else {
-      /* set stop time in last track */
-      stop =
-          gst_util_uint64_scale_round (offset, GST_SECOND,
-          flacparse->samplerate);
-      gst_toc_entry_set_start_stop_times (cur_entry, start, stop);
-    }
-  }
-
-  /* send data as TOC */
-  if (!flacparse->toc)
-    flacparse->toc = toc;
-
-  gst_buffer_unmap (buffer, &map);
-  return TRUE;
-
-error:
-  GST_ERROR_OBJECT (flacparse, "Error reading data");
-  gst_buffer_unmap (buffer, &map);
-  return FALSE;
-}
-
 static gboolean
 gst_flac_parse_handle_picture (GstFlacParse * flacparse, GstBuffer * buffer)
 {
@@ -1128,7 +999,7 @@ gst_flac_parse_handle_picture (GstFlacParse * flacparse, GstBuffer * buffer)
       map.data + gst_byte_reader_get_pos (&reader), img_len, img_type);
 
   if (gst_tag_list_is_empty (flacparse->tags)) {
-    gst_tag_list_unref (flacparse->tags);
+    gst_tag_list_free (flacparse->tags);
     flacparse->tags = NULL;
   }
 
@@ -1254,16 +1125,6 @@ gst_flac_parse_handle_headers (GstFlacParse * flacparse)
     gst_buffer_unmap (header, &map);
   }
 
-  /* at least this one we can generate easily
-   * to provide full headers downstream */
-  if (vorbiscomment == NULL && streaminfo != NULL) {
-    GST_DEBUG_OBJECT (flacparse,
-        "missing vorbiscomment header; generating dummy");
-    vorbiscomment = gst_flac_parse_generate_vorbiscomment (flacparse);
-    flacparse->headers = g_list_insert (flacparse->headers, vorbiscomment,
-        g_list_index (flacparse->headers, streaminfo) + 1);
-  }
-
   if (marker == NULL || streaminfo == NULL || vorbiscomment == NULL) {
     GST_WARNING_OBJECT (flacparse,
         "missing header %p %p %p, muxing into container "
@@ -1352,56 +1213,10 @@ push_headers:
   return res;
 }
 
-/* empty vorbiscomment */
-static GstBuffer *
-gst_flac_parse_generate_vorbiscomment (GstFlacParse * flacparse)
-{
-  GstTagList *taglist = gst_tag_list_new_empty ();
-  guchar header[4];
-  guint size;
-  GstBuffer *vorbiscomment;
-  GstMapInfo map;
-
-  header[0] = 0x84;             /* is_last = 1; type = 4; */
-
-  vorbiscomment =
-      gst_tag_list_to_vorbiscomment_buffer (taglist, header,
-      sizeof (header), NULL);
-  gst_tag_list_unref (taglist);
-
-  gst_buffer_map (vorbiscomment, &map, GST_MAP_WRITE);
-
-  /* Get rid of framing bit */
-  if (map.data[map.size - 1] == 1) {
-    GstBuffer *sub;
-
-    sub =
-        gst_buffer_copy_region (vorbiscomment, GST_BUFFER_COPY_ALL, 0,
-        map.size - 1);
-    gst_buffer_unmap (vorbiscomment, &map);
-    gst_buffer_unref (vorbiscomment);
-    vorbiscomment = sub;
-    gst_buffer_map (vorbiscomment, &map, GST_MAP_WRITE);
-  }
-
-  size = map.size - 4;
-  map.data[1] = ((size & 0xFF0000) >> 16);
-  map.data[2] = ((size & 0x00FF00) >> 8);
-  map.data[3] = (size & 0x0000FF);
-  gst_buffer_unmap (vorbiscomment, &map);
-
-  GST_BUFFER_TIMESTAMP (vorbiscomment) = GST_CLOCK_TIME_NONE;
-  GST_BUFFER_DURATION (vorbiscomment) = GST_CLOCK_TIME_NONE;
-  GST_BUFFER_OFFSET (vorbiscomment) = 0;
-  GST_BUFFER_OFFSET_END (vorbiscomment) = 0;
-
-  return vorbiscomment;
-}
-
 static gboolean
 gst_flac_parse_generate_headers (GstFlacParse * flacparse)
 {
-  GstBuffer *marker, *streaminfo;
+  GstBuffer *marker, *streaminfo, *vorbiscomment;
   GstMapInfo map;
 
   marker = gst_buffer_new_and_alloc (4);
@@ -1470,8 +1285,46 @@ gst_flac_parse_generate_headers (GstFlacParse * flacparse)
   GST_BUFFER_OFFSET_END (streaminfo) = 0;
   flacparse->headers = g_list_append (flacparse->headers, streaminfo);
 
-  flacparse->headers = g_list_append (flacparse->headers,
-      gst_flac_parse_generate_vorbiscomment (flacparse));
+  /* empty vorbiscomment */
+  {
+    GstTagList *taglist = gst_tag_list_new_empty ();
+    guchar header[4];
+    guint size;
+
+    header[0] = 0x84;           /* is_last = 1; type = 4; */
+
+    vorbiscomment =
+        gst_tag_list_to_vorbiscomment_buffer (taglist, header,
+        sizeof (header), NULL);
+    gst_tag_list_free (taglist);
+
+    gst_buffer_map (vorbiscomment, &map, GST_MAP_WRITE);
+
+    /* Get rid of framing bit */
+    if (map.data[map.size - 1] == 1) {
+      GstBuffer *sub;
+
+      sub =
+          gst_buffer_copy_region (vorbiscomment, GST_BUFFER_COPY_ALL, 0,
+          map.size - 1);
+      gst_buffer_unmap (vorbiscomment, &map);
+      gst_buffer_unref (vorbiscomment);
+      vorbiscomment = sub;
+      gst_buffer_map (vorbiscomment, &map, GST_MAP_WRITE);
+    }
+
+    size = map.size - 4;
+    map.data[1] = ((size & 0xFF0000) >> 16);
+    map.data[2] = ((size & 0x00FF00) >> 8);
+    map.data[3] = (size & 0x0000FF);
+    gst_buffer_unmap (vorbiscomment, &map);
+
+    GST_BUFFER_TIMESTAMP (vorbiscomment) = GST_CLOCK_TIME_NONE;
+    GST_BUFFER_DURATION (vorbiscomment) = GST_CLOCK_TIME_NONE;
+    GST_BUFFER_OFFSET (vorbiscomment) = 0;
+    GST_BUFFER_OFFSET_END (vorbiscomment) = 0;
+    flacparse->headers = g_list_append (flacparse->headers, vorbiscomment);
+  }
 
   return TRUE;
 }
@@ -1528,16 +1381,13 @@ gst_flac_parse_parse_frame (GstBaseParse * parse, GstBaseParseFrame * frame,
         if (!gst_flac_parse_handle_vorbiscomment (flacparse, sbuffer))
           goto cleanup;
         break;
-      case 5:                  /* CUESHEET */
-        if (!gst_flac_parse_handle_cuesheet (flacparse, sbuffer))
-          goto cleanup;
-        break;
       case 6:                  /* PICTURE */
         if (!gst_flac_parse_handle_picture (flacparse, sbuffer))
           goto cleanup;
         break;
       case 1:                  /* PADDING */
       case 2:                  /* APPLICATION */
+      case 5:                  /* CUESHEET */
       default:                 /* RESERVED */
         break;
     }
@@ -1656,12 +1506,6 @@ gst_flac_parse_pre_push_frame (GstBaseParse * parse, GstBaseParseFrame * frame)
         gst_event_new_tag (flacparse->tags));
     flacparse->tags = NULL;
   }
-  /* Push toc */
-  if (flacparse->toc) {
-    gst_pad_push_event (GST_BASE_PARSE_SRC_PAD (flacparse),
-        gst_event_new_toc (flacparse->toc, FALSE));
-    flacparse->toc = NULL;
-  }
 
   frame->flags |= GST_BASE_PARSE_FRAME_FLAG_CLIP;
 
@@ -1706,9 +1550,10 @@ gst_flac_parse_get_sink_caps (GstBaseParse * parse, GstCaps * filter)
   GstCaps *peercaps, *templ;
   GstCaps *res;
 
+  /* FIXME: handle filter caps */
   templ = gst_pad_get_pad_template_caps (GST_BASE_PARSE_SINK_PAD (parse));
-  peercaps = gst_pad_get_allowed_caps (GST_BASE_PARSE_SRC_PAD (parse));
 
+  peercaps = gst_pad_get_allowed_caps (GST_BASE_PARSE_SRC_PAD (parse));
   if (peercaps) {
     guint i, n;
 
@@ -1723,24 +1568,10 @@ gst_flac_parse_get_sink_caps (GstBaseParse * parse, GstCaps * filter)
 
     res = gst_caps_intersect_full (peercaps, templ, GST_CAPS_INTERSECT_FIRST);
     gst_caps_unref (peercaps);
-
-    /* Append the template caps because we still want to accept
-     * caps without any fields in the case upstream does not
-     * know anything.
-     */
-    gst_caps_append (res, templ);
+    gst_caps_unref (templ);
   } else {
     res = templ;
   }
 
-  if (filter) {
-    GstCaps *intersection;
-
-    intersection =
-        gst_caps_intersect_full (filter, res, GST_CAPS_INTERSECT_FIRST);
-    gst_caps_unref (res);
-    res = intersection;
-  }
-
   return res;
 }
