@@ -565,7 +565,7 @@ gst_wavpack_parse_handle_frame (GstBaseParse * parse,
         if (!gst_wavpack_get_channel_positions (chans, mask, pos)) {
           GST_WARNING_OBJECT (wvparse, "Failed to determine channel layout");
         } else {
-          gst_audio_channel_positions_to_mask (pos, chans, FALSE, &gmask);
+          gst_audio_channel_positions_to_mask (pos, chans, &gmask);
           if (gmask)
             gst_caps_set_simple (caps,
                 "channel-mask", GST_TYPE_BITMASK, gmask, NULL);
@@ -614,9 +614,10 @@ gst_wavpack_parse_get_sink_caps (GstBaseParse * parse, GstCaps * filter)
   GstCaps *peercaps, *templ;
   GstCaps *res;
 
+  /* FIXME: handle filter caps */
   templ = gst_pad_get_pad_template_caps (GST_BASE_PARSE_SINK_PAD (parse));
-  peercaps = gst_pad_get_allowed_caps (GST_BASE_PARSE_SRC_PAD (parse));
 
+  peercaps = gst_pad_get_allowed_caps (GST_BASE_PARSE_SRC_PAD (parse));
   if (peercaps) {
     guint i, n;
 
@@ -631,24 +632,10 @@ gst_wavpack_parse_get_sink_caps (GstBaseParse * parse, GstCaps * filter)
 
     res = gst_caps_intersect_full (peercaps, templ, GST_CAPS_INTERSECT_FIRST);
     gst_caps_unref (peercaps);
-
-    /* Append the template caps because we still want to accept
-     * caps without any fields in the case upstream does not
-     * know anything.
-     */
-    gst_caps_append (res, templ);
+    gst_caps_unref (templ);
   } else {
     res = templ;
   }
 
-  if (filter) {
-    GstCaps *intersection;
-
-    intersection =
-        gst_caps_intersect_full (filter, res, GST_CAPS_INTERSECT_FIRST);
-    gst_caps_unref (res);
-    res = intersection;
-  }
-
   return res;
 }
