@@ -34,7 +34,7 @@
  * <title>Example launch lines</title>
  * <para>(write everything in one line, without the backslash characters)</para>
  * |[
- * gst-launch-1.0 videotestsrc num-buffers=250 \
+ * gst-launch videotestsrc num-buffers=250 \
  * ! 'video/x-raw,format=(string)I420,width=320,height=240,framerate=(fraction)25/1' \
  * ! queue ! mux. \
  * audiotestsrc num-buffers=440 ! audioconvert \
@@ -44,7 +44,7 @@
  * with a test picture and an uncompressed audio stream containing a
  * test sound.
  * |[
- * gst-launch-1.0 videotestsrc num-buffers=250 \
+ * gst-launch videotestsrc num-buffers=250 \
  * ! 'video/x-raw,format=(string)I420,width=320,height=240,framerate=(fraction)25/1' \
  * ! xvidenc ! queue ! mux. \
  * audiotestsrc num-buffers=440 ! audioconvert ! 'audio/x-raw,rate=44100,channels=2' \
@@ -106,6 +106,14 @@ static GstStaticPadTemplate video_sink_factory =
         "height = (int) [ 16, 4096 ], "
         "framerate = (fraction) [ 0, MAX ], "
         "divxversion = (int) [ 3, 5 ]; "
+        "video/x-xvid, "
+        "width = (int) [ 16, 4096 ], "
+        "height = (int) [ 16, 4096 ], "
+        "framerate = (fraction) [ 0, MAX ]; "
+        "video/x-3ivx, "
+        "width = (int) [ 16, 4096 ], "
+        "height = (int) [ 16, 4096 ], "
+        "framerate = (fraction) [ 0, MAX ]; "
         "video/x-msmpeg, "
         "width = (int) [ 16, 4096 ], "
         "height = (int) [ 16, 4096 ], "
@@ -346,7 +354,7 @@ gst_avi_mux_reset (GstAviMux * avimux)
   avimux->codec_data_size = 0;
 
   if (avimux->tags_snap) {
-    gst_tag_list_unref (avimux->tags_snap);
+    gst_tag_list_free (avimux->tags_snap);
     avimux->tags_snap = NULL;
   }
 
@@ -503,6 +511,10 @@ gst_avi_mux_vidsink_set_caps (GstPad * pad, GstCaps * vscaps)
           avipad->vids.compression = GST_MAKE_FOURCC ('D', 'X', '5', '0');
           break;
       }
+    } else if (!strcmp (mimetype, "video/x-xvid")) {
+      avipad->vids.compression = GST_MAKE_FOURCC ('X', 'V', 'I', 'D');
+    } else if (!strcmp (mimetype, "video/x-3ivx")) {
+      avipad->vids.compression = GST_MAKE_FOURCC ('3', 'I', 'V', '2');
     } else if (gst_structure_has_name (structure, "video/x-msmpeg")) {
       gint msmpegversion;
 
@@ -757,11 +769,11 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
     switch (fmt) {
       case GST_AUDIO_FORMAT_U8:
         avipad->auds.blockalign = 8;
-        avipad->auds.bits_per_sample = 8;
+        avipad->auds.size = 8;
         break;
       case GST_AUDIO_FORMAT_S16:
         avipad->auds.blockalign = 16;
-        avipad->auds.bits_per_sample = 16;
+        avipad->auds.size = 16;
         break;
       default:
         goto refuse_caps;
@@ -777,7 +789,7 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
     /* set some defaults */
     avipad->auds.blockalign = 1;
     avipad->auds.av_bps = 0;
-    avipad->auds.bits_per_sample = 16;
+    avipad->auds.size = 16;
 
     if (!strcmp (mimetype, "audio/mpeg")) {
       gint mpegversion;
@@ -847,12 +859,12 @@ gst_avi_mux_audsink_set_caps (GstPad * pad, GstCaps * vscaps)
       avipad->auds.format = GST_RIFF_WAVE_FORMAT_A52;
     } else if (!strcmp (mimetype, "audio/x-alaw")) {
       avipad->auds.format = GST_RIFF_WAVE_FORMAT_ALAW;
-      avipad->auds.bits_per_sample = 8;
+      avipad->auds.size = 8;
       avipad->auds.blockalign = avipad->auds.channels;
       avipad->auds.av_bps = avipad->auds.blockalign * avipad->auds.rate;
     } else if (!strcmp (mimetype, "audio/x-mulaw")) {
       avipad->auds.format = GST_RIFF_WAVE_FORMAT_MULAW;
-      avipad->auds.bits_per_sample = 8;
+      avipad->auds.size = 8;
       avipad->auds.blockalign = avipad->auds.channels;
       avipad->auds.av_bps = avipad->auds.blockalign * avipad->auds.rate;
     } else if (!strcmp (mimetype, "audio/x-wma")) {
@@ -968,7 +980,7 @@ gst_avi_mux_request_new_pad (GstElement * element,
   g_free (name);
 
   avipad->collect = gst_collect_pads_add_pad (avimux->collect,
-      newpad, sizeof (GstAviCollectData), NULL, TRUE);
+      newpad, sizeof (GstAviCollectData));
   ((GstAviCollectData *) (avipad->collect))->avipad = avipad;
 
   if (!gst_element_add_pad (element, newpad))
@@ -1100,32 +1112,18 @@ gst_avi_mux_write_tag (const GstTagList * list, const gchar * tag,
     0, NULL}
   };
   gint n;
-  gchar *str = NULL;
+  gchar *str;
   GstByteWriter *bw = data;
   guint chunk;
 
   for (n = 0; rifftags[n].fcc != 0; n++) {
-    if (!strcmp (rifftags[n].tag, tag)) {
-      if (rifftags[n].fcc == GST_RIFF_INFO_ICRD) {
-        GDate *date;
-        /* special case for the date tag */
-        if (gst_tag_list_get_date (list, tag, &date)) {
-          str =
-              g_strdup_printf ("%04d:%02d:%02d", g_date_get_year (date),
-              g_date_get_month (date), g_date_get_day (date));
-          g_date_free (date);
-        }
-      } else {
-        gst_tag_list_get_string (list, tag, &str);
-      }
-      if (str) {
-        chunk = gst_avi_mux_start_chunk (bw, NULL, rifftags[n].fcc);
-        gst_byte_writer_put_string (bw, str);
-        gst_avi_mux_end_chunk (bw, chunk);
-        g_free (str);
-        str = NULL;
-        break;
-      }
+    if (!strcmp (rifftags[n].tag, tag) &&
+        gst_tag_list_get_string (list, tag, &str) && str) {
+      chunk = gst_avi_mux_start_chunk (bw, NULL, rifftags[n].fcc);
+      gst_byte_writer_put_string (bw, str);
+      gst_avi_mux_end_chunk (bw, chunk);
+      g_free (str);
+      break;
     }
   }
 }
@@ -1286,7 +1284,7 @@ gst_avi_mux_riff_get_avi_header (GstAviMux * avimux)
       hdl &= gst_byte_writer_put_uint32_le (&bw, audpad->auds.rate);
       hdl &= gst_byte_writer_put_uint32_le (&bw, audpad->auds.av_bps);
       hdl &= gst_byte_writer_put_uint16_le (&bw, audpad->auds.blockalign);
-      hdl &= gst_byte_writer_put_uint16_le (&bw, audpad->auds.bits_per_sample);
+      hdl &= gst_byte_writer_put_uint16_le (&bw, audpad->auds.size);
       hdl &= gst_byte_writer_put_uint16_le (&bw, codec_size);
       if (audpad->auds_codec_data) {
         gst_buffer_map (audpad->auds_codec_data, &map, GST_MAP_READ);
