@@ -25,7 +25,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch-1.0 -v filesrc location=/path/to/flv ! flvdemux ! audioconvert ! autoaudiosink
+ * gst-launch -v filesrc location=/path/to/flv ! flvdemux ! audioconvert ! autoaudiosink
  * ]| This pipeline demuxes an FLV file and outputs the contained raw audio streams.
  * </refsect2>
  */
@@ -312,7 +312,7 @@ gst_flv_demux_parse_metadata_item (GstFlvDemux * demux, GstByteReader * reader,
   GST_DEBUG_OBJECT (demux, "tag name %s, tag type %d", tag_name, tag_type);
 
   switch (tag_type) {
-    case 0:                    /* Double */
+    case 0:                    // Double
     {                           /* Use a union to read the uint64 and then as a double */
       gdouble d = 0;
 
@@ -344,7 +344,7 @@ gst_flv_demux_parse_metadata_item (GstFlvDemux * demux, GstByteReader * reader,
 
       break;
     }
-    case 1:                    /* Boolean */
+    case 1:                    // Boolean
     {
       guint8 b = 0;
 
@@ -357,7 +357,7 @@ gst_flv_demux_parse_metadata_item (GstFlvDemux * demux, GstByteReader * reader,
 
       break;
     }
-    case 2:                    /* String */
+    case 2:                    // String
     {
       gchar *s = NULL;
 
@@ -395,7 +395,7 @@ gst_flv_demux_parse_metadata_item (GstFlvDemux * demux, GstByteReader * reader,
 
       break;
     }
-    case 3:                    /* Object */
+    case 3:                    // Object
     {
       gboolean end_of_object_marker = FALSE;
 
@@ -411,7 +411,7 @@ gst_flv_demux_parse_metadata_item (GstFlvDemux * demux, GstByteReader * reader,
 
       break;
     }
-    case 8:                    /* ECMA array */
+    case 8:                    // ECMA array
     {
       guint32 nb_elems = 0;
       gboolean end_of_object_marker = FALSE;
@@ -434,7 +434,7 @@ gst_flv_demux_parse_metadata_item (GstFlvDemux * demux, GstByteReader * reader,
 
       break;
     }
-    case 9:                    /* End marker */
+    case 9:                    // End marker
     {
       GST_DEBUG_OBJECT (demux, "end marker ?");
       if (tag_name[0] == '\0') {
@@ -446,7 +446,7 @@ gst_flv_demux_parse_metadata_item (GstFlvDemux * demux, GstByteReader * reader,
 
       break;
     }
-    case 10:                   /* Array */
+    case 10:                   // Array
     {
       guint32 nb_elems = 0;
 
@@ -499,7 +499,7 @@ gst_flv_demux_parse_metadata_item (GstFlvDemux * demux, GstByteReader * reader,
 
       break;
     }
-    case 11:                   /* Date */
+    case 11:                   // Date
     {
       gdouble d = 0;
       gint16 i = 0;
@@ -642,7 +642,6 @@ gst_flv_demux_audio_negotiate (GstFlvDemux * demux, guint32 codec_tag,
   gchar *codec_name = NULL;
   gboolean ret = FALSE;
   guint adjusted_rate = rate;
-  gchar *stream_id;
 
   switch (codec_tag) {
     case 1:
@@ -733,11 +732,6 @@ gst_flv_demux_audio_negotiate (GstFlvDemux * demux, guint32 codec_tag,
         demux->audio_codec_data, NULL);
   }
 
-  stream_id =
-      gst_pad_create_stream_id (demux->audio_pad, GST_ELEMENT_CAST (demux),
-      "audio");
-  gst_pad_push_event (demux->audio_pad, gst_event_new_stream_start (stream_id));
-  g_free (stream_id);
   ret = gst_pad_set_caps (demux->audio_pad, caps);
 
   if (G_LIKELY (ret)) {
@@ -802,7 +796,6 @@ gst_flv_demux_push_tags (GstFlvDemux * demux)
   if (demux->taglist) {
     GST_DEBUG_OBJECT (demux, "pushing tags out %" GST_PTR_FORMAT,
         demux->taglist);
-    gst_tag_list_set_scope (demux->taglist, GST_TAG_SCOPE_GLOBAL);
     gst_flv_demux_push_src_event (demux, gst_event_new_tag (demux->taglist));
     demux->taglist = gst_tag_list_new_empty ();
     demux->push_tags = FALSE;
@@ -1128,7 +1121,6 @@ gst_flv_demux_video_negotiate (GstFlvDemux * demux, guint32 codec_tag)
   gboolean ret = FALSE;
   GstCaps *caps = NULL;
   gchar *codec_name = NULL;
-  gchar *stream_id;
 
   /* Generate caps for that pad */
   switch (codec_tag) {
@@ -1185,10 +1177,6 @@ gst_flv_demux_video_negotiate (GstFlvDemux * demux, guint32 codec_tag)
         demux->video_codec_data, NULL);
   }
 
-  stream_id =
-      gst_pad_create_stream_id (demux->video_pad, GST_ELEMENT_CAST (demux),
-      "video");
-  gst_pad_push_event (demux->video_pad, gst_event_new_stream_start (stream_id));
   ret = gst_pad_set_caps (demux->video_pad, caps);
 
   if (G_LIKELY (ret)) {
@@ -2469,17 +2457,12 @@ pause:
           gst_element_post_message (GST_ELEMENT_CAST (demux),
               gst_message_new_segment_done (GST_OBJECT_CAST (demux),
                   GST_FORMAT_TIME, stop));
-          gst_flv_demux_push_src_event (demux,
-              gst_event_new_segment_done (GST_FORMAT_TIME, stop));
         } else {                /* Reverse playback */
           GST_LOG_OBJECT (demux, "Sending segment done, at beginning of "
               "segment");
           gst_element_post_message (GST_ELEMENT_CAST (demux),
               gst_message_new_segment_done (GST_OBJECT_CAST (demux),
                   GST_FORMAT_TIME, demux->segment.start));
-          gst_flv_demux_push_src_event (demux,
-              gst_event_new_segment_done (GST_FORMAT_TIME,
-                  demux->segment.start));
         }
       } else {
         /* normal playback, send EOS to all linked pads */
@@ -2489,10 +2472,7 @@ pause:
         }
 
         GST_LOG_OBJECT (demux, "Sending EOS, at end of stream");
-        if (!demux->audio_pad && !demux->video_pad)
-          GST_ELEMENT_ERROR (demux, STREAM, FAILED,
-              ("Internal data stream error."), ("Got EOS before any data"));
-        else if (!gst_flv_demux_push_src_event (demux, gst_event_new_eos ()))
+        if (!gst_flv_demux_push_src_event (demux, gst_event_new_eos ()))
           GST_WARNING_OBJECT (demux, "failed pushing EOS on streams");
       }
     } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_EOS) {
@@ -2844,7 +2824,7 @@ exit:
     gst_pad_pause_task (demux->sinkpad);
   } else {
     gst_pad_start_task (demux->sinkpad,
-        (GstTaskFunction) gst_flv_demux_loop, demux->sinkpad, NULL);
+        (GstTaskFunction) gst_flv_demux_loop, demux->sinkpad);
   }
 
   GST_PAD_STREAM_UNLOCK (demux->sinkpad);
@@ -2875,8 +2855,7 @@ gst_flv_demux_sink_activate (GstPad * sinkpad, GstObject * parent)
     goto activate_push;
   }
 
-  pull_mode = gst_query_has_scheduling_mode_with_flags (query,
-      GST_PAD_MODE_PULL, GST_SCHEDULING_FLAG_SEEKABLE);
+  pull_mode = gst_query_has_scheduling_mode (query, GST_PAD_MODE_PULL);
   gst_query_unref (query);
 
   if (!pull_mode)
@@ -2910,7 +2889,7 @@ gst_flv_demux_sink_activate_mode (GstPad * sinkpad, GstObject * parent,
       if (active) {
         demux->random_access = TRUE;
         res = gst_pad_start_task (sinkpad, (GstTaskFunction) gst_flv_demux_loop,
-            sinkpad, NULL);
+            sinkpad);
       } else {
         demux->random_access = FALSE;
         res = gst_pad_stop_task (sinkpad);
@@ -2957,19 +2936,13 @@ gst_flv_demux_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
         gst_index_commit (index, demux->index_id);
         gst_object_unref (index);
       }
-
-      if (!demux->audio_pad && !demux->video_pad)
-        GST_ELEMENT_ERROR (demux, STREAM, FAILED,
-            ("Internal data stream error."), ("Got EOS before any data"));
-      else {
-        if (!demux->no_more_pads) {
-          gst_element_no_more_pads (GST_ELEMENT (demux));
-          demux->no_more_pads = TRUE;
-        }
-
-        if (!gst_flv_demux_push_src_event (demux, event))
-          GST_WARNING_OBJECT (demux, "failed pushing EOS on streams");
+      if (!demux->no_more_pads) {
+        gst_element_no_more_pads (GST_ELEMENT (demux));
+        demux->no_more_pads = TRUE;
       }
+
+      if (!gst_flv_demux_push_src_event (demux, event))
+        GST_WARNING_OBJECT (demux, "failed pushing EOS on streams");
       ret = TRUE;
       break;
     }
@@ -2996,10 +2969,6 @@ gst_flv_demux_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
         demux->video_need_segment = TRUE;
         ret = TRUE;
         gst_event_unref (event);
-        if (demux->new_seg_event) {
-          gst_event_unref (demux->new_seg_event);
-          demux->new_seg_event = NULL;
-        }
       }
       break;
     }
@@ -3250,7 +3219,7 @@ gst_flv_demux_dispose (GObject * object)
   }
 
   if (demux->taglist) {
-    gst_tag_list_unref (demux->taglist);
+    gst_tag_list_free (demux->taglist);
     demux->taglist = NULL;
   }
 
