@@ -131,7 +131,7 @@ gst_icydemux_reset (GstICYDemux * icydemux)
   gst_icydemux_remove_srcpad (icydemux);
 
   if (icydemux->cached_tags) {
-    gst_tag_list_unref (icydemux->cached_tags);
+    gst_tag_list_free (icydemux->cached_tags);
     icydemux->cached_tags = NULL;
   }
 
@@ -231,16 +231,14 @@ gst_icydemux_add_srcpad (GstICYDemux * icydemux, GstCaps * new_caps)
     g_return_val_if_fail (icydemux->srcpad != NULL, FALSE);
 
     gst_pad_use_fixed_caps (icydemux->srcpad);
-    gst_pad_set_active (icydemux->srcpad, TRUE);
 
-    if (icydemux->src_caps) {
-      if (!gst_pad_set_caps (icydemux->srcpad, icydemux->src_caps))
-        GST_WARNING_OBJECT (icydemux, "Failed to set caps on src pad");
-    }
+    if (icydemux->src_caps)
+      gst_pad_set_caps (icydemux->srcpad, icydemux->src_caps);
 
     GST_DEBUG_OBJECT (icydemux, "Adding src pad with caps %" GST_PTR_FORMAT,
         icydemux->src_caps);
 
+    gst_pad_set_active (icydemux->srcpad, TRUE);
     if (!(gst_element_add_pad (GST_ELEMENT (icydemux), icydemux->srcpad)))
       return FALSE;
     gst_element_no_more_pads (GST_ELEMENT (icydemux));
@@ -287,7 +285,7 @@ gst_icydemux_tag_found (GstICYDemux * icydemux, GstTagList * tags)
   } else {
     gst_tag_list_insert (icydemux->cached_tags, tags,
         GST_TAG_MERGE_REPLACE_ALL);
-    gst_tag_list_unref (tags);
+    gst_tag_list_free (tags);
   }
 
   return TRUE;
@@ -341,7 +339,7 @@ gst_icydemux_parse_and_send_tags (GstICYDemux * icydemux)
   if (!gst_tag_list_is_empty (tags))
     gst_icydemux_tag_found (icydemux, tags);
   else
-    gst_tag_list_unref (tags);
+    gst_tag_list_free (tags);
 }
 
 static gboolean
@@ -614,6 +612,9 @@ gst_icydemux_send_tag_event (GstICYDemux * icydemux, GstTagList * tags)
 {
   GstEvent *event;
 
+  gst_element_post_message (GST_ELEMENT (icydemux),
+      gst_message_new_tag (GST_OBJECT (icydemux), gst_tag_list_copy (tags)));
+
   event = gst_event_new_tag (tags);
   GST_EVENT_TIMESTAMP (event) = 0;
 
