@@ -72,8 +72,6 @@ typedef struct _GstMatroskaDemux {
   GstClockTime             last_stop_end;
   GstClockTime             stream_start_time;
 
-  /* Stop time for reverse playback */
-  GstClockTime             to_time;
   GstEvent                *new_segment;
 
   /* some state saving */
@@ -81,8 +79,6 @@ typedef struct _GstMatroskaDemux {
   guint64                  cluster_offset;
   guint64                  first_cluster_offset;
   guint64                  next_cluster_offset;
-  GstClockTime             requested_seek_time;
-  guint64                  seek_offset;
 
   /* index stuff */
   gboolean                 seekable;
