@@ -30,18 +30,19 @@
  * directories). The results will be sorted.
  *
  * <refsect2>
- * <title>Example launch lines</title>
+ * <title>Example launch line</title>
  * |[
- * gst-launch-1.0 splitfilesrc location="/path/to/part-*.mpg" ! decodebin ! ...
+ * gst-launch splitfilesrc location="/path/to/part-*.mpg" ! decodebin ! ... \
  * ]| Plays the different parts as if they were one single MPEG file.
- * |[
- * gst-launch-1.0 playbin uri="splitfile://path/to/foo.avi.*"
- * ]| Plays the different parts as if they were one single AVI file.
  * </refsect2>
  *
  * Since: 0.10.31
  */
 
+/* TODO:
+ *  - implement splitfile:// URI handler?
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -64,8 +65,6 @@ enum
 
 #define DEFAULT_LOCATION NULL
 
-static void gst_split_file_src_uri_handler_init (gpointer g_iface,
-    gpointer iface_data);
 static void gst_split_file_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_split_file_src_get_property (GObject * object, guint prop_id,
@@ -89,10 +88,7 @@ GST_STATIC_PAD_TEMPLATE ("src",
 GST_DEBUG_CATEGORY_STATIC (splitfilesrc_debug);
 #define GST_CAT_DEFAULT splitfilesrc_debug
 
-
-G_DEFINE_TYPE_WITH_CODE (GstSplitFileSrc, gst_split_file_src, GST_TYPE_BASE_SRC,
-    G_IMPLEMENT_INTERFACE (GST_TYPE_URI_HANDLER,
-        gst_split_file_src_uri_handler_init));
+G_DEFINE_TYPE (GstSplitFileSrc, gst_split_file_src, GST_TYPE_BASE_SRC);
 
 #ifdef G_OS_WIN32
 #define WIN32_BLURB " Location string must be in UTF-8 encoding (on Windows)."
@@ -186,25 +182,6 @@ gst_split_file_src_get_size (GstBaseSrc * basesrc, guint64 * size)
   return TRUE;
 }
 
-static void
-gst_split_file_src_set_location (GstSplitFileSrc * src, const char *location)
-{
-  GST_OBJECT_LOCK (src);
-  g_free (src->location);
-
-  if (location != NULL && g_str_has_prefix (location, "splitfile://"))
-    src->location = gst_uri_get_location (location);
-  else
-    src->location = g_strdup (location);
-#ifdef G_OS_WIN32
-  if (!g_utf8_validate (src->location, -1, NULL)) {
-    g_warning ("splitfilesrc 'location' property must be in UTF-8 "
-        "encoding on Windows");
-  }
-#endif
-  GST_OBJECT_UNLOCK (src);
-}
-
 static void
 gst_split_file_src_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -213,7 +190,16 @@ gst_split_file_src_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case PROP_LOCATION:
-      gst_split_file_src_set_location (src, g_value_get_string (value));
+      GST_OBJECT_LOCK (src);
+      g_free (src->location);
+      src->location = g_value_dup_string (value);
+#ifdef G_OS_WIN32
+      if (!g_utf8_validate (src->location, -1, NULL)) {
+        g_warning ("splitfilesrc 'location' property must be in UTF-8 "
+            "encoding on Windows");
+      }
+#endif
+      GST_OBJECT_UNLOCK (src);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -617,55 +603,3 @@ cancelled:
     return GST_FLOW_FLUSHING;
   }
 }
-
-static guint
-gst_split_file_src_uri_get_type (GType type)
-{
-  return GST_URI_SRC;
-}
-
-static const gchar *const *
-gst_split_file_src_uri_get_protocols (GType type)
-{
-  static const gchar *protocols[] = { "splitfile", NULL };
-
-  return (const gchar * const *) protocols;
-}
-
-static gchar *
-gst_split_file_src_uri_get_uri (GstURIHandler * handler)
-{
-  GstSplitFileSrc *src = GST_SPLIT_FILE_SRC (handler);
-  gchar *ret;
-
-  GST_OBJECT_LOCK (src);
-  if (src->location != NULL)
-    ret = g_strdup_printf ("splitfile://%s", src->location);
-  else
-    ret = NULL;
-  GST_OBJECT_UNLOCK (src);
-
-  return ret;
-}
-
-static gboolean
-gst_split_file_src_uri_set_uri (GstURIHandler * handler, const gchar * uri,
-    GError ** error)
-{
-  GstSplitFileSrc *src = GST_SPLIT_FILE_SRC (handler);
-
-  gst_split_file_src_set_location (src, uri);
-
-  return TRUE;
-}
-
-static void
-gst_split_file_src_uri_handler_init (gpointer g_iface, gpointer iface_data)
-{
-  GstURIHandlerInterface *iface = (GstURIHandlerInterface *) g_iface;
-
-  iface->get_type = gst_split_file_src_uri_get_type;
-  iface->get_protocols = gst_split_file_src_uri_get_protocols;
-  iface->get_uri = gst_split_file_src_uri_get_uri;
-  iface->set_uri = gst_split_file_src_uri_set_uri;
-}
