@@ -69,6 +69,11 @@ GST_DEBUG_CATEGORY_STATIC (rtpjpegpay_debug);
  */
 #define QUANT_PREFIX_LEN     3
 
+/*
+ * DEFAULT_BUFFER_LIST:
+ *
+ */
+#define DEFAULT_BUFFER_LIST            FALSE
 
 typedef enum _RtpJpegMarker RtpJpegMarker;
 
@@ -113,6 +118,7 @@ enum
   PROP_0,
   PROP_JPEG_QUALITY,
   PROP_JPEG_TYPE,
+  PROP_BUFFER_LIST,
   PROP_LAST
 };
 
@@ -270,6 +276,11 @@ gst_rtp_jpeg_pay_class_init (GstRtpJPEGPayClass * klass)
           "Default JPEG Type, overwritten by SOF when present", 0, 255,
           DEFAULT_JPEG_TYPE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BUFFER_LIST,
+      g_param_spec_boolean ("buffer-list", "Buffer List",
+          "Use Buffer Lists",
+          DEFAULT_BUFFER_LIST, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   GST_DEBUG_CATEGORY_INIT (rtpjpegpay_debug, "rtpjpegpay", 0,
       "Motion JPEG RTP Payloader");
 }
@@ -280,6 +291,7 @@ gst_rtp_jpeg_pay_init (GstRtpJPEGPay * pay)
   pay->quality = DEFAULT_JPEG_QUALITY;
   pay->quant = DEFAULT_JPEG_QUANT;
   pay->type = DEFAULT_JPEG_TYPE;
+  pay->buffer_list = DEFAULT_BUFFER_LIST;
 }
 
 static gboolean
@@ -405,8 +417,7 @@ no_table:
 
 static gboolean
 gst_rtp_jpeg_pay_read_sof (GstRtpJPEGPay * pay, const guint8 * data,
-    guint size, guint * offset, CompInfo info[], RtpQuantTable tables[],
-    gulong tables_elements)
+    guint size, guint * offset, CompInfo info[])
 {
   guint sof_size, off;
   guint width, height, infolen;
@@ -484,18 +495,8 @@ gst_rtp_jpeg_pay_read_sof (GstRtpJPEGPay * pay, const guint8 * data,
 
   /* the other components are free to use any quant table but they have to
    * have the same table id */
-  if (info[1].qt != info[2].qt) {
-    /* Some MJPG (like the one from the Logitech C-920 camera) uses different
-     * quant tables for component 1 and 2 but both tables contain the exact
-     * same data, so we could consider them as being the same tables */
-    if (!(info[1].qt < tables_elements &&
-            info[2].qt < tables_elements &&
-            tables[info[1].qt].size > 0 &&
-            tables[info[1].qt].size == tables[info[2].qt].size &&
-            memcmp (tables[info[1].qt].data, tables[info[2].qt].data,
-                tables[info[1].qt].size) == 0))
-      goto invalid_comp;
-  }
+  if (info[1].qt != info[2].qt)
+    goto invalid_comp;
 
   return TRUE;
 
@@ -647,8 +648,7 @@ gst_rtp_jpeg_pay_handle_buffer (GstRTPBasePayload * basepayload,
         offset += gst_rtp_jpeg_pay_header_size (data, offset);
         break;
       case JPEG_MARKER_SOF:
-        if (!gst_rtp_jpeg_pay_read_sof (pay, data, size, &offset, info, tables,
-                G_N_ELEMENTS (tables)))
+        if (!gst_rtp_jpeg_pay_read_sof (pay, data, size, &offset, info))
           goto invalid_format;
         sof_found = TRUE;
         break;
@@ -732,7 +732,9 @@ gst_rtp_jpeg_pay_handle_buffer (GstRTPBasePayload * basepayload,
 
   GST_LOG_OBJECT (pay, "quant_data size %u", quant_data_size);
 
-  list = gst_buffer_list_new ();
+  if (pay->buffer_list) {
+    list = gst_buffer_list_new ();
+  }
 
   bytes_left = sizeof (jpeg_header) + quant_data_size + size;
 
@@ -816,8 +818,14 @@ gst_rtp_jpeg_pay_handle_buffer (GstRTPBasePayload * basepayload,
 
     GST_BUFFER_TIMESTAMP (outbuf) = timestamp;
 
-    /* and add to list */
-    gst_buffer_list_insert (list, -1, outbuf);
+    if (pay->buffer_list) {
+      /* and add to list */
+      gst_buffer_list_insert (list, -1, outbuf);
+    } else {
+      ret = gst_rtp_base_payload_push (basepayload, outbuf);
+      if (ret != GST_FLOW_OK)
+        break;
+    }
 
     bytes_left -= payload_size;
     offset += payload_size;
@@ -825,8 +833,10 @@ gst_rtp_jpeg_pay_handle_buffer (GstRTPBasePayload * basepayload,
   }
   while (!frame_done);
 
-  /* push the whole buffer list at once */
-  ret = gst_rtp_base_payload_push_list (basepayload, list);
+  if (pay->buffer_list) {
+    /* push the whole buffer list at once */
+    ret = gst_rtp_base_payload_push_list (basepayload, list);
+  }
 
   gst_buffer_unmap (buffer, &map);
   gst_buffer_unref (buffer);
@@ -881,6 +891,10 @@ gst_rtp_jpeg_pay_set_property (GObject * object, guint prop_id,
       rtpjpegpay->type = g_value_get_int (value);
       GST_DEBUG_OBJECT (object, "type = %d", rtpjpegpay->type);
       break;
+    case PROP_BUFFER_LIST:
+      rtpjpegpay->buffer_list = g_value_get_boolean (value);
+      GST_DEBUG_OBJECT (object, "buffer_list = %d", rtpjpegpay->buffer_list);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -902,6 +916,9 @@ gst_rtp_jpeg_pay_get_property (GObject * object, guint prop_id,
     case PROP_JPEG_TYPE:
       g_value_set_int (value, rtpjpegpay->type);
       break;
+    case PROP_BUFFER_LIST:
+      g_value_set_boolean (value, rtpjpegpay->buffer_list);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
