@@ -178,9 +178,9 @@ struct _GstRTSPSrc {
   GstClockTime     base_time;
 
   /* UDP mode loop */
-  gint             pending_cmd;
-  gint             busy_cmd;
+  gint             loop_cmd;
   gboolean         ignore_timeout;
+  gboolean         waiting;
   gboolean         open_error;
 
   /* mutex for protecting state changes */
