@@ -31,7 +31,7 @@
  * <refsect2>
  * <title>Sample pipelines</title>
  * |[
- * gst-launch-1.0 -v videotestsrc pattern=1 ! smpte name=s border=20000 type=234 duration=2000000000 ! videoconvert ! ximagesink videotestsrc ! s.
+ * gst-launch -v videotestsrc pattern=1 ! smpte name=s border=20000 type=234 duration=2000000000 ! ffmpegcolorspace ! ximagesink videotestsrc ! s.
  * ]| A pipeline to demonstrate the smpte transition.
  * It shows a pinwheel transition a from a snow videotestsrc to an smpte
  * pattern videotestsrc. The transition will take 2 seconds to complete. The
@@ -84,6 +84,7 @@ enum
 #define DEFAULT_PROP_TYPE	1
 #define DEFAULT_PROP_BORDER	0
 #define DEFAULT_PROP_DEPTH	16
+#define DEFAULT_PROP_FPS	0.
 #define DEFAULT_PROP_DURATION	GST_SECOND
 #define DEFAULT_PROP_INVERT   FALSE
 
@@ -93,6 +94,7 @@ enum
   PROP_TYPE,
   PROP_BORDER,
   PROP_DEPTH,
+  PROP_FPS,
   PROP_DURATION,
   PROP_INVERT,
   PROP_LAST,
@@ -183,6 +185,11 @@ gst_smpte_class_init (GstSMPTEClass * klass)
       g_param_spec_enum ("type", "Type", "The type of transition to use",
           GST_TYPE_SMPTE_TRANSITION_TYPE, DEFAULT_PROP_TYPE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_FPS,
+      g_param_spec_float ("fps", "FPS",
+          "Frames per second if no input files are given (deprecated)", 0.,
+          G_MAXFLOAT, DEFAULT_PROP_FPS,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BORDER,
       g_param_spec_int ("border", "Border",
           "The border width of the transition", 0, G_MAXINT,
@@ -281,6 +288,9 @@ gst_smpte_setcaps (GstPad * pad, GstCaps * caps)
   smpte->fps_num = GST_VIDEO_INFO_FPS_N (&vinfo);
   smpte->fps_denom = GST_VIDEO_INFO_FPS_D (&vinfo);
 
+  /* for backward compat, we store these here */
+  smpte->fps = ((gdouble) smpte->fps_num) / smpte->fps_denom;
+
   /* figure out the duration in frames */
   smpte->end_position = gst_util_uint64_scale (smpte->duration,
       smpte->fps_num, GST_SECOND * smpte->fps_denom);
@@ -356,10 +366,11 @@ gst_smpte_init (GstSMPTE * smpte)
       GST_DEBUG_FUNCPTR (gst_smpte_sink_event), smpte);
 
   gst_collect_pads_add_pad (smpte->collect, smpte->sinkpad1,
-      sizeof (GstCollectData), NULL, TRUE);
+      sizeof (GstCollectData));
   gst_collect_pads_add_pad (smpte->collect, smpte->sinkpad2,
-      sizeof (GstCollectData), NULL, TRUE);
+      sizeof (GstCollectData));
 
+  smpte->fps = DEFAULT_PROP_FPS;
   smpte->type = DEFAULT_PROP_TYPE;
   smpte->border = DEFAULT_PROP_BORDER;
   smpte->depth = DEFAULT_PROP_DEPTH;
@@ -587,6 +598,9 @@ gst_smpte_set_property (GObject * object, guint prop_id,
     case PROP_BORDER:
       smpte->border = g_value_get_int (value);
       break;
+    case PROP_FPS:
+      smpte->fps = g_value_get_float (value);
+      break;
     case PROP_DEPTH:
       smpte->depth = g_value_get_int (value);
       break;
@@ -614,6 +628,9 @@ gst_smpte_get_property (GObject * object, guint prop_id,
     case PROP_TYPE:
       g_value_set_enum (value, smpte->type);
       break;
+    case PROP_FPS:
+      g_value_set_float (value, smpte->fps);
+      break;
     case PROP_BORDER:
       g_value_set_int (value, smpte->border);
       break;
