@@ -468,6 +468,11 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
     mem = gst_buffer_get_memory (buffer, i);
     gst_memory_map (mem, &map[i], GST_MAP_READ);
 
+    if (map[i].size > UDP_MAX_SIZE) {
+      GST_WARNING ("Attempting to send a UDP packet larger than maximum "
+          "size (%" G_GSIZE_FORMAT " > %d)", map[i].size, UDP_MAX_SIZE);
+    }
+
     vec[i].buffer = map[i].data;
     vec[i].size = map[i].size;
 
@@ -481,11 +486,6 @@ gst_multiudpsink_render (GstBaseSink * bsink, GstBuffer * buffer)
   g_mutex_lock (&sink->client_lock);
   GST_LOG_OBJECT (bsink, "about to send %" G_GSIZE_FORMAT " bytes", size);
 
-  if (size > UDP_MAX_SIZE) {
-    GST_WARNING_OBJECT (bsink, "Attempting to send a UDP packet larger than "
-        "maximum size (%" G_GSIZE_FORMAT " > %d)", size, UDP_MAX_SIZE);
-  }
-
   no_clients = 0;
   num = 0;
   for (clients = sink->clients; clients; clients = g_list_next (clients)) {
@@ -537,16 +537,10 @@ no_data:
   }
 send_error:
   {
-    GstFlowReturn res = GST_FLOW_ERROR;
-
     g_mutex_unlock (&sink->client_lock);
     GST_DEBUG ("got send error %s", err->message);
-
-    if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_CANCELLED))
-      res = GST_FLOW_FLUSHING;
-
     g_clear_error (&err);
-    return res;
+    return GST_FLOW_ERROR;
   }
 }
 
