@@ -89,16 +89,16 @@
  * <refsect2>
  * <title>Examples</title>
  * |[
- * gst-launch-1.0 -v udpsrc ! fakesink dump=1
+ * gst-launch -v udpsrc ! fakesink dump=1
  * ]| A pipeline to read from the default port and dump the udp packets.
  * To actually generate udp packets on the default port one can use the
  * udpsink element. When running the following pipeline in another terminal, the
  * above mentioned pipeline should dump data packets to the console.
  * |[
- * gst-launch-1.0 -v audiotestsrc ! udpsink
+ * gst-launch -v audiotestsrc ! udpsink
  * ]|
  * |[
- * gst-launch-1.0 -v udpsrc port=0 ! fakesink
+ * gst-launch -v udpsrc port=0 ! fakesink
  * ]| read udp packets from a free port.
  * </refsect2>
  *
@@ -954,7 +954,7 @@ gst_udpsrc_stop (GstBaseSrc * bsrc)
 
       if (!g_socket_leave_multicast_group (src->used_socket,
               g_inet_socket_address_get_address (src->addr), FALSE,
-              src->multi_iface, &err)) {
+              src->multi_iface, NULL)) {
         GST_ERROR_OBJECT (src, "Failed to leave multicast group: %s",
             err->message);
         g_clear_error (&err);
