@@ -42,7 +42,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch-1.0 -v videotestsrc ! videocrop top=42 left=1 right=4 bottom=0 ! ximagesink
+ * gst-launch -v videotestsrc ! videocrop top=42 left=1 right=4 bottom=0 ! ximagesink
  * ]|
  * </refsect2>
  */
@@ -80,7 +80,7 @@ enum
 #define VIDEO_CROP_CAPS                                \
   GST_VIDEO_CAPS_MAKE ("{ RGBx, xRGB, BGRx, xBGR, "    \
       "RGBA, ARGB, BGRA, ABGR, RGB, BGR, AYUV, YUY2, " \
-      "YVYU, UYVY, I420, RGB16, RGB15, GRAY8 }")
+      "YVYU, UYVY, Y800, I420, RGB16, RGB15, GRAY8 }")
 
 static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -266,7 +266,7 @@ gst_video_crop_get_image_details_from_caps (GstVideoCrop * vcrop,
           details->macro_y_off = 0;
         }
         break;
-      case GST_VIDEO_FORMAT_GRAY8:
+      case GST_VIDEO_FORMAT_Y800:
         details->packing = VIDEO_CROP_PIXEL_FORMAT_PACKED_SIMPLE;
         break;
       case GST_VIDEO_FORMAT_I420:
@@ -609,7 +609,12 @@ gst_video_crop_transform_caps (GstBaseTransform * trans,
     gst_caps_append_structure (other_caps, new_structure);
   }
 
-  if (!gst_caps_is_empty (other_caps) && filter_caps) {
+  if (gst_caps_is_empty (other_caps)) {
+    gst_caps_unref (other_caps);
+    other_caps = NULL;
+  }
+
+  if (other_caps && filter_caps) {
     GstCaps *tmp = gst_caps_intersect_full (filter_caps, other_caps,
         GST_CAPS_INTERSECT_FIRST);
     gst_caps_replace (&other_caps, tmp);
