@@ -36,7 +36,7 @@
  * <refsect2>
  * <title>Sample pipelines</title>
  * |[
- * gst-launch-1.0 \
+ * gst-launch-0.11 \
  *   videotestsrc pattern=1 ! \
  *   video/x-raw,format=AYUV,framerate=\(fraction\)10/1,width=100,height=100 ! \
  *   videobox border-alpha=0 top=-70 bottom=-70 right=-220 ! \
@@ -52,21 +52,21 @@
  * video test source behind and the checker pattern under it. Note that the
  * framerate of the output video is 10 frames per second.
  * |[
- * gst-launch-1.0 videotestsrc pattern=1 ! \
+ * gst-launch-0.11 videotestsrc pattern=1 ! \
  *   video/x-raw, framerate=\(fraction\)10/1, width=100, height=100 ! \
  *   videomixer name=mix ! videoconvert ! ximagesink \
  *   videotestsrc !  \
  *   video/x-raw, framerate=\(fraction\)5/1, width=320, height=240 ! mix.
  * ]| A pipeline to demostrate bgra mixing. (This does not demonstrate alpha blending). 
  * |[
- * gst-launch-1.0 videotestsrc pattern=1 ! \
+ * gst-launch-0.11 videotestsrc pattern=1 ! \
  *   video/x-raw,format =I420, framerate=\(fraction\)10/1, width=100, height=100 ! \
  *   videomixer name=mix ! videoconvert ! ximagesink \
  *   videotestsrc ! \
  *   video/x-raw,format=I420, framerate=\(fraction\)5/1, width=320, height=240 ! mix.
  * ]| A pipeline to test I420
  * |[
- * gst-launch-1.0 videomixer name=mixer sink_1::alpha=0.5 sink_1::xpos=50 sink_1::ypos=50 ! \
+ * gst-launch-0.11 videomixer name=mixer sink_1::alpha=0.5 sink_1::xpos=50 sink_1::ypos=50 ! \
  *   videoconvert ! ximagesink \
  *   videotestsrc pattern=snow timestamp-offset=3000000000 ! \
  *   "video/x-raw,format=AYUV,width=640,height=480,framerate=(fraction)30/1" ! \
@@ -104,8 +104,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_videomixer2_debug);
   (g_mutex_unlock(GST_VIDEO_MIXER2_GET_LOCK (mix)))
 
 #define FORMATS " { AYUV, BGRA, ARGB, RGBA, ABGR, Y444, Y42B, YUY2, UYVY, "\
-                "   YVYU, I420, YV12, NV12, NV21, Y41B, RGB, BGR, xRGB, xBGR, "\
-                "   RGBx, BGRx } "
+                "   YVYU, I420, YV12, Y41B, RGB, BGR, xRGB, xBGR, RGBx, BGRx } "
 
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -228,7 +227,6 @@ gst_videomixer2_update_src_caps (GstVideoMixer2 * mix)
         mix->nframes = 0;
       }
     }
-    gst_video_info_init (&info);
     gst_video_info_set_format (&info, GST_VIDEO_INFO_FORMAT (&mix->info),
         best_width, best_height);
     info.fps_n = best_fps_n;
@@ -326,13 +324,16 @@ beach:
 }
 
 static GstCaps *
-gst_videomixer2_pad_sink_getcaps (GstPad * pad, GstVideoMixer2 * mix,
+gst_videomixer2_pad_sink_getcaps (GstPad * pad, GstObject * parent,
     GstCaps * filter)
 {
+  GstVideoMixer2 *mix;
   GstCaps *srccaps;
   GstStructure *s;
   gint i, n;
 
+  mix = GST_VIDEO_MIXER2 (parent);
+
   srccaps = gst_pad_get_current_caps (GST_PAD (mix->srcpad));
   if (srccaps == NULL)
     srccaps = gst_pad_get_pad_template_caps (GST_PAD (mix->srcpad));
@@ -356,14 +357,16 @@ gst_videomixer2_pad_sink_getcaps (GstPad * pad, GstVideoMixer2 * mix,
 }
 
 static gboolean
-gst_videomixer2_pad_sink_acceptcaps (GstPad * pad, GstVideoMixer2 * mix,
+gst_videomixer2_pad_sink_acceptcaps (GstPad * pad, GstObject * parent,
     GstCaps * caps)
 {
   gboolean ret;
+  GstVideoMixer2 *mix;
   GstCaps *accepted_caps;
   gint i, n;
   GstStructure *s;
 
+  mix = GST_VIDEO_MIXER2 (parent);
   GST_DEBUG_OBJECT (pad, "%" GST_PTR_FORMAT, caps);
 
   accepted_caps = gst_pad_get_current_caps (GST_PAD (mix->srcpad));
@@ -394,10 +397,9 @@ gst_videomixer2_pad_sink_acceptcaps (GstPad * pad, GstVideoMixer2 * mix,
 }
 
 static gboolean
-gst_videomixer2_sink_query (GstCollectPads * pads, GstCollectData * cdata,
-    GstQuery * query, GstVideoMixer2 * mix)
+gst_videomixer2_pad_sink_query (GstPad * pad, GstObject * parent,
+    GstQuery * query)
 {
-  GstVideoMixer2Pad *pad = GST_VIDEO_MIXER2_PAD (cdata->pad);
   gboolean ret = FALSE;
 
   switch (GST_QUERY_TYPE (query)) {
@@ -406,7 +408,7 @@ gst_videomixer2_sink_query (GstCollectPads * pads, GstCollectData * cdata,
       GstCaps *filter, *caps;
 
       gst_query_parse_caps (query, &filter);
-      caps = gst_videomixer2_pad_sink_getcaps (GST_PAD (pad), mix, filter);
+      caps = gst_videomixer2_pad_sink_getcaps (pad, parent, filter);
       gst_query_set_caps_result (query, caps);
       gst_caps_unref (caps);
       ret = TRUE;
@@ -417,13 +419,13 @@ gst_videomixer2_sink_query (GstCollectPads * pads, GstCollectData * cdata,
       GstCaps *caps;
 
       gst_query_parse_accept_caps (query, &caps);
-      ret = gst_videomixer2_pad_sink_acceptcaps (GST_PAD (pad), mix, caps);
+      ret = gst_videomixer2_pad_sink_acceptcaps (pad, parent, caps);
       gst_query_set_accept_caps_result (query, ret);
       ret = TRUE;
       break;
     }
     default:
-      ret = gst_collect_pads_query_default (pads, cdata, query, FALSE);
+      ret = gst_pad_query_default (pad, parent, query);
       break;
   }
   return ret;
@@ -523,6 +525,10 @@ gst_videomixer2_pad_class_init (GstVideoMixer2PadClass * klass)
 static void
 gst_videomixer2_pad_init (GstVideoMixer2Pad * mixerpad)
 {
+  /* setup some pad functions */
+  gst_pad_set_query_function (GST_PAD (mixerpad),
+      gst_videomixer2_pad_sink_query);
+
   mixerpad->zorder = DEFAULT_PAD_ZORDER;
   mixerpad->xpos = DEFAULT_PAD_XPOS;
   mixerpad->ypos = DEFAULT_PAD_YPOS;
@@ -812,7 +818,7 @@ gst_videomixer2_blend_buffers (GstVideoMixer2 * mix,
   guint outsize;
   BlendFunction composite;
   GstVideoFrame outframe;
-  static GstAllocationParams params = { 0, 15, 0, 0, };
+  static GstAllocationParams params = { 0, 0, 0, 15, };
 
   outsize = GST_VIDEO_INFO_SIZE (&mix->info);
 
@@ -1501,20 +1507,6 @@ gst_videomixer2_src_setcaps (GstPad * pad, GstVideoMixer2 * mix, GstCaps * caps)
       mix->fill_color = gst_video_mixer_fill_color_yv12;
       ret = TRUE;
       break;
-    case GST_VIDEO_FORMAT_NV12:
-      mix->blend = gst_video_mixer_blend_nv12;
-      mix->overlay = mix->blend;
-      mix->fill_checker = gst_video_mixer_fill_checker_nv12;
-      mix->fill_color = gst_video_mixer_fill_color_nv12;
-      ret = TRUE;
-      break;
-    case GST_VIDEO_FORMAT_NV21:
-      mix->blend = gst_video_mixer_blend_nv21;
-      mix->overlay = mix->blend;
-      mix->fill_checker = gst_video_mixer_fill_checker_nv21;
-      mix->fill_color = gst_video_mixer_fill_color_nv21;
-      ret = TRUE;
-      break;
     case GST_VIDEO_FORMAT_Y41B:
       mix->blend = gst_video_mixer_blend_y41b;
       mix->overlay = mix->blend;
@@ -1790,7 +1782,7 @@ gst_videomixer2_request_new_pad (GstElement * element,
     mixpad->alpha = DEFAULT_PAD_ALPHA;
 
     mixcol = (GstVideoMixer2Collect *)
-        gst_collect_pads_add_pad (mix->collect, GST_PAD (mixpad),
+        gst_collect_pads_add_pad_full (mix->collect, GST_PAD (mixpad),
         sizeof (GstVideoMixer2Collect),
         (GstCollectDataDestroyNotify) gst_videomixer2_collect_free, TRUE);
 
@@ -1813,7 +1805,7 @@ gst_videomixer2_request_new_pad (GstElement * element,
 
   /* add the pad to the element */
   gst_element_add_pad (element, GST_PAD (mixpad));
-  gst_child_proxy_child_added (GST_CHILD_PROXY (mix), G_OBJECT (mixpad),
+  gst_child_proxy_child_added (G_OBJECT (mix), G_OBJECT (mixpad),
       GST_OBJECT_NAME (mixpad));
 
   return GST_PAD (mixpad);
@@ -1837,7 +1829,7 @@ gst_videomixer2_release_pad (GstElement * element, GstPad * pad)
   mixpad = GST_VIDEO_MIXER2_PAD (pad);
 
   mix->sinkpads = g_slist_remove (mix->sinkpads, pad);
-  gst_child_proxy_child_removed (GST_CHILD_PROXY (mix), G_OBJECT (mixpad),
+  gst_child_proxy_child_removed (G_OBJECT (mix), G_OBJECT (mixpad),
       GST_OBJECT_NAME (mixpad));
   mix->numpads--;
 
@@ -1997,8 +1989,6 @@ gst_videomixer2_init (GstVideoMixer2 * mix)
       mix);
   gst_collect_pads_set_event_function (mix->collect,
       (GstCollectPadsEventFunction) gst_videomixer2_sink_event, mix);
-  gst_collect_pads_set_query_function (mix->collect,
-      (GstCollectPadsQueryFunction) gst_videomixer2_sink_query, mix);
   gst_collect_pads_set_clip_function (mix->collect,
       (GstCollectPadsClipFunction) gst_videomixer2_sink_clip, mix);
 
