@@ -21,6 +21,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/* FIXME 0.11: suppress warnings for deprecated API such as GValueArray
+ * with newer GLib versions (>= 2.31.0) */
+#define GLIB_DISABLE_DEPRECATION_WARNINGS
+
 #include <gst/check/gstcheck.h>
 #include <gst/tag/tag.h>
 
@@ -100,6 +104,8 @@ static guint8 granuleshift;
 
 static GstPad *srcpad, *sinkpad;
 
+static GList *events;
+
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
@@ -116,14 +122,15 @@ static GstBuffer *
 buffer_new (const gchar * buffer_data, guint size)
 {
   GstBuffer *buffer;
+  guint8 *data;
 
-  GstCaps *caps;
+  data = g_malloc (size);
+  memcpy (data, buffer_data, size);
 
-  buffer = gst_buffer_new_and_alloc (size);
-  memcpy (GST_BUFFER_DATA (buffer), buffer_data, size);
-  caps = gst_caps_from_string (SRC_CAPS);
-  gst_buffer_set_caps (buffer, caps);
-  gst_caps_unref (caps);
+  if (data)
+    buffer = gst_buffer_new_wrapped (data, size);
+  else
+    buffer = gst_buffer_new ();
 
   return buffer;
 }
@@ -135,13 +142,38 @@ buffer_unref (void *buffer, void *user_data)
   gst_buffer_unref (GST_BUFFER (buffer));
 }
 
+static void
+event_unref (void *event, void *user_data)
+{
+  ASSERT_OBJECT_REFCOUNT (event, "event", 1);
+  gst_event_unref (GST_EVENT (event));
+}
+
+static gboolean
+cmmldec_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
+{
+  GST_LOG ("received event %" GST_PTR_FORMAT, event);
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_TAG:
+      events = g_list_append (events, event);
+      break;
+    default:
+      gst_event_unref (event);
+      break;
+  }
+
+  return TRUE;
+}
+
 static void
 setup_cmmldec (void)
 {
   GST_DEBUG ("setup_cmmldec");
   cmmldec = gst_check_setup_element ("cmmldec");
-  srcpad = gst_check_setup_src_pad (cmmldec, &srctemplate, NULL);
-  sinkpad = gst_check_setup_sink_pad (cmmldec, &sinktemplate, NULL);
+  srcpad = gst_check_setup_src_pad (cmmldec, &srctemplate);
+  sinkpad = gst_check_setup_sink_pad (cmmldec, &sinktemplate);
+  gst_pad_set_event_function (sinkpad, cmmldec_sink_event);
   gst_pad_set_active (srcpad, TRUE);
   gst_pad_set_active (sinkpad, TRUE);
 
@@ -174,6 +206,11 @@ teardown_cmmldec (void)
   gst_check_teardown_src_pad (cmmldec);
   gst_check_teardown_sink_pad (cmmldec);
   gst_check_teardown_element (cmmldec);
+
+  /* sticky stuff cleared above first */
+  g_list_foreach (events, event_unref, NULL);
+  g_list_free (events);
+  events = NULL;
 }
 
 static void
@@ -181,6 +218,7 @@ check_output_buffer_is_equal (const gchar * name,
     const gchar * data, gint refcount)
 {
   GstBuffer *buffer;
+  GstMapInfo map;
 
   if (current_buf == NULL)
     current_buf = buffers;
@@ -190,11 +228,13 @@ check_output_buffer_is_equal (const gchar * name,
   fail_unless (current_buf != NULL);
 
   buffer = GST_BUFFER (current_buf->data);
+  gst_buffer_map (buffer, &map, GST_MAP_READ);
 
   ASSERT_OBJECT_REFCOUNT (buffer, name, refcount);
-  fail_unless (memcmp (GST_BUFFER_DATA (buffer), data,
-          GST_BUFFER_SIZE (buffer)) == 0,
-      "'%s' (%s) is not equal to (%s)", name, GST_BUFFER_DATA (buffer), data);
+  fail_unless (memcmp (map.data, data, map.size) == 0,
+      "'%s' (%s) is not equal to (%s)", name, map.data, data);
+
+  gst_buffer_unmap (buffer, &map);
 }
 
 static GstFlowReturn
@@ -207,32 +247,39 @@ push_data (const gchar * name, const gchar * data, gint size, gint64 granulepos)
   return gst_pad_push (srcpad, buffer);
 }
 
+static void
+push_caps (void)
+{
+  GstCaps *caps;
+
+  caps = gst_caps_from_string (SRC_CAPS);
+  fail_unless (gst_pad_set_caps (srcpad, caps));
+  gst_caps_unref (caps);
+}
+
 static GObject *
 cmml_tag_message_pop (GstBus * bus, const gchar * tag)
 {
-  GstMessage *message;
-
   GstTagList *taglist;
-
   const GValue *value;
+  GObject *obj = NULL;
+  GstEvent *event;
 
-  GObject *obj;
-
-  message = gst_bus_poll (bus, GST_MESSAGE_TAG, 0);
-  if (message == NULL)
+  if (!events)
     return NULL;
 
-  gst_message_parse_tag (message, &taglist);
+  event = (GstEvent *) events->data;
+  events = g_list_delete_link (events, events);
+  gst_event_parse_tag (event, &taglist);
+
   value = gst_tag_list_get_value_index (taglist, tag, 0);
-  if (value == NULL) {
-    gst_message_unref (message);
-    gst_tag_list_free (taglist);
-    return NULL;
-  }
+  if (value == NULL)
+    goto exit;
 
   obj = g_value_dup_object (value);
-  gst_message_unref (message);
-  gst_tag_list_free (taglist);
+
+exit:
+  gst_event_unref (event);
 
   return obj;
 }
@@ -246,6 +293,8 @@ check_headers (void)
 
   GValueArray *meta;
 
+  push_caps ();
+
   /* push the ident header */
   flow = push_data ("ident-header", IDENT_HEADER, IDENT_HEADER_SIZE, 0);
   fail_unless_equals_flow_return (flow, GST_FLOW_OK);
@@ -371,6 +420,8 @@ GST_END_TEST;
 
 GST_START_TEST (test_preamble_no_pi)
 {
+  push_caps ();
+
   flow = push_data ("ident-header", IDENT_HEADER, IDENT_HEADER_SIZE, 0);
   fail_unless_equals_flow_return (flow, GST_FLOW_OK);
   fail_unless_equals_int (g_list_length (buffers), 0);
@@ -531,6 +582,8 @@ GST_START_TEST (test_weird_input)
 {
   const gchar *bad_xml = "<?xml version=\"1.0\"?><a><b></a>";
 
+  push_caps ();
+
   /* malformed ident header */
   flow = push_data ("bad-ident-header", "CMML\0\0\0\0garbage", 15, 0);
   fail_unless_equals_flow_return (flow, GST_FLOW_ERROR);
@@ -569,7 +622,7 @@ GST_START_TEST (test_sink_query_convert)
   granulepos = keyindex + keyoffset;
 
   fail_unless (gst_pad_query_convert (GST_PAD_PEER (srcpad),
-          GST_FORMAT_DEFAULT, granulepos, &dstfmt, &dstval));
+          GST_FORMAT_DEFAULT, granulepos, dstfmt, &dstval));
 
   fail_unless (dstfmt == GST_FORMAT_TIME);
   /* fail unless dstval == index + offset */
