@@ -66,7 +66,6 @@
  * sink pads we create; otherwise we always have to do get_pad, get_peer, and
  * then remove references in every test function */
 static GstPad *mysrcpad, *mysinkpad;
-static GstBus *mybus;
 
 /* Mapping from supported sample rates to the correct result gain for the
  * following test waveform: 20 * 512 samples with a quarter-full amplitude of
@@ -147,27 +146,6 @@ static GstStaticPadTemplate srctemplate = GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS (RG_ANALYSIS_CAPS_TEMPLATE_STRING)
     );
 
-static gboolean
-mysink_event_func (GstPad * pad, GstObject * parent, GstEvent * event)
-{
-  GST_LOG_OBJECT (pad, "%s event: %" GST_PTR_FORMAT,
-      GST_EVENT_TYPE_NAME (event), event);
-
-  /* a sink would post tag events as messages, so do the same here,
-   * esp. since we're polling on the bus waiting for TAG messages.. */
-  if (GST_EVENT_TYPE (event) == GST_EVENT_TAG) {
-    GstTagList *taglist;
-
-    gst_event_parse_tag (event, &taglist);
-
-    gst_bus_post (mybus, gst_message_new_tag (GST_OBJECT (mysinkpad),
-            gst_tag_list_copy (taglist)));
-  }
-
-  gst_event_unref (event);
-  return TRUE;
-}
-
 static GstElement *
 setup_rganalysis (void)
 {
@@ -178,14 +156,13 @@ setup_rganalysis (void)
   analysis = gst_check_setup_element ("rganalysis");
   mysrcpad = gst_check_setup_src_pad (analysis, &srctemplate);
   mysinkpad = gst_check_setup_sink_pad (analysis, &sinktemplate);
-  gst_pad_set_event_function (mysinkpad, mysink_event_func);
   gst_pad_set_active (mysrcpad, TRUE);
   gst_pad_set_active (mysinkpad, TRUE);
 
   bus = gst_bus_new ();
   gst_element_set_bus (analysis, bus);
-
-  mybus = bus;                  /* keep ref */
+  /* gst_element_set_bus does not steal a reference. */
+  gst_object_unref (bus);
 
   return analysis;
 }
@@ -199,9 +176,6 @@ cleanup_rganalysis (GstElement * element)
   g_list_free (buffers);
   buffers = NULL;
 
-  gst_object_unref (mybus);
-  mybus = NULL;
-
   /* The bus owns references to the element: */
   gst_element_set_bus (element, NULL);
 
@@ -220,43 +194,15 @@ set_playing_state (GstElement * element)
       "Could not set state to PLAYING");
 }
 
-static void
-send_flush_events (GstElement * element)
-{
-  gboolean res;
-  GstPad *pad;
-
-  pad = gst_element_get_static_pad (element, "sink");
-  res = gst_pad_send_event (pad, gst_event_new_flush_start ());
-  fail_unless (res, "flush-start even not handledt");
-  res = gst_pad_send_event (pad, gst_event_new_flush_stop (TRUE));
-  fail_unless (res, "flush-stop event not handled");
-  gst_object_unref (pad);
-}
-
-static void
-send_segment_event (GstElement * element)
-{
-  GstSegment segment;
-  gboolean res;
-  GstPad *pad;
-
-  pad = gst_element_get_static_pad (element, "sink");
-  gst_segment_init (&segment, GST_FORMAT_TIME);
-  res = gst_pad_send_event (pad, gst_event_new_segment (&segment));
-  fail_unless (res, "SEGMENT event not handled");
-  gst_object_unref (pad);
-}
-
 static void
 send_eos_event (GstElement * element)
 {
   GstBus *bus = gst_element_get_bus (element);
   GstPad *pad = gst_element_get_static_pad (element, "sink");
-  gboolean res;
+  GstEvent *event = gst_event_new_eos ();
 
-  res = gst_pad_send_event (pad, gst_event_new_eos ());
-  fail_unless (res, "EOS event not handled");
+  fail_unless (gst_pad_send_event (pad, event),
+      "Cannot send EOS event: Not handled.");
 
   /* There is no sink element, so _we_ post the EOS message on the bus here.  Of
    * course we generate any EOS ourselves, but this allows us to poll for the
@@ -703,27 +649,20 @@ GST_START_TEST (test_no_buffer_album_1)
   set_playing_state (element);
 
   /* Single track: */
-  send_segment_event (element);
   send_eos_event (element);
   poll_eos (element);
 
   /* First album: */
   g_object_set (element, "num-tracks", 3, NULL);
 
-  send_flush_events (element);
-  send_segment_event (element);
   send_eos_event (element);
   poll_eos (element);
   fail_unless_num_tracks (element, 2);
 
-  send_flush_events (element);
-  send_segment_event (element);
   send_eos_event (element);
   poll_eos (element);
   fail_unless_num_tracks (element, 1);
 
-  send_flush_events (element);
-  send_segment_event (element);
   send_eos_event (element);
   poll_eos (element);
   fail_unless_num_tracks (element, 0);
@@ -731,21 +670,15 @@ GST_START_TEST (test_no_buffer_album_1)
   /* Second album: */
   g_object_set (element, "num-tracks", 2, NULL);
 
-  send_flush_events (element);
-  send_segment_event (element);
   send_eos_event (element);
   poll_eos (element);
   fail_unless_num_tracks (element, 1);
 
-  send_flush_events (element);
-  send_segment_event (element);
   send_eos_event (element);
   poll_eos (element);
   fail_unless_num_tracks (element, 0);
 
   /* Single track: */
-  send_flush_events (element);
-  send_segment_event (element);
   send_eos_event (element);
   poll_eos (element);
   fail_unless_num_tracks (element, 0);
@@ -767,7 +700,6 @@ GST_START_TEST (test_no_buffer_album_2)
 
   /* No buffer for the first track. */
 
-  send_segment_event (element);
   send_eos_event (element);
   /* No tags should be posted, there was nothing to analyze: */
   poll_eos (element);
@@ -775,8 +707,6 @@ GST_START_TEST (test_no_buffer_album_2)
 
   /* A test waveform with known gain result as second track: */
 
-  send_flush_events (element);
-  send_segment_event (element);
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_mono (&accumulator, 44100, 512,
             0.25));
@@ -786,13 +716,11 @@ GST_START_TEST (test_no_buffer_album_2)
   fail_unless_track_gain (tag_list, -6.20);
   /* Album is not finished yet: */
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 1);
 
   /* No buffer for the last track. */
 
-  send_flush_events (element);
-  send_segment_event (element);
   send_eos_event (element);
 
   tag_list = poll_tags (element);
@@ -800,7 +728,7 @@ GST_START_TEST (test_no_buffer_album_2)
   fail_unless_album_gain (tag_list, -6.20);
   /* No track tags should be posted, as there was no data for it: */
   fail_if_track_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 0);
 
   cleanup_rganalysis (element);
@@ -815,7 +743,6 @@ GST_START_TEST (test_empty_buffers)
   set_playing_state (element);
 
   /* Single track: */
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (44100, 0, 0.0, 0.0));
   send_eos_event (element);
   poll_eos (element);
@@ -823,15 +750,11 @@ GST_START_TEST (test_empty_buffers)
   /* First album: */
   g_object_set (element, "num-tracks", 2, NULL);
 
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (44100, 0, 0.0, 0.0));
   send_eos_event (element);
   poll_eos (element);
   fail_unless_num_tracks (element, 1);
 
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (44100, 0, 0.0, 0.0));
   send_eos_event (element);
   poll_eos (element);
@@ -839,16 +762,12 @@ GST_START_TEST (test_empty_buffers)
 
   /* Second album, with a single track: */
   g_object_set (element, "num-tracks", 1, NULL);
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (44100, 0, 0.0, 0.0));
   send_eos_event (element);
   poll_eos (element);
   fail_unless_num_tracks (element, 0);
 
   /* Single track: */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (44100, 0, 0.0, 0.0));
   send_eos_event (element);
   poll_eos (element);
@@ -875,30 +794,25 @@ GST_START_TEST (test_peak_float)
   GstTagList *tag_list;
 
   set_playing_state (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 512, -1.369, 0.0));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 1.369);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Swapped channels. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 512, 0.0, -1.369));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 1.369);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Mono. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_mono (8000, 512, -1.369));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 1.369);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -913,57 +827,46 @@ GST_START_TEST (test_peak_int16_16)
   set_playing_state (element);
 
   /* Half amplitude. */
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 1 << 14, 0));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.5);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Swapped channels. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 0, 1 << 14));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.5);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Mono. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_mono (8000, 16, 512, 1 << 14));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.5);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Half amplitude, negative variant. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, -1 << 14, 0));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.5);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Swapped channels. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 0, -1 << 14));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.5);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Mono. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_mono (8000, 16, 512, -1 << 14));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.5);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
 
   /* Now check for correct normalization of the peak value: Sample
@@ -971,59 +874,47 @@ GST_START_TEST (test_peak_int16_16)
    * highest positive amplitude we do not reach 1.0, only for
    * -32768! */
 
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 32767, 0));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 32767. / 32768.);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Swapped channels. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 0, 32767));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 32767. / 32768.);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Mono. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_mono (8000, 16, 512, 32767));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 32767. / 32768.);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
 
   /* Negative variant, reaching 1.0. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, -32768, 0));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 1.0);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Swapped channels. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_stereo (8000, 16, 512, 0, -32768));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 1.0);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   /* Mono. */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_int16_mono (8000, 16, 512, -32768));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 1.0);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -1038,68 +929,57 @@ GST_START_TEST (test_peak_album)
   g_object_set (element, "num-tracks", 2, NULL);
   set_playing_state (element);
 
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 1.0, 0.0));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 1.0);
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 1);
 
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.0, 0.5));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.5);
   fail_unless_album_peak (tag_list, 1.0);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 0);
 
   /* Try a second album: */
   g_object_set (element, "num-tracks", 3, NULL);
 
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.4, 0.4));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.4);
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 2);
 
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.45, 0.45));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.45);
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 1);
 
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.2, 0.2));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.2);
   fail_unless_album_peak (tag_list, 0.45);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 0);
 
   /* And now a single track, not in album mode (num-tracks is 0
    * now): */
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.1, 0.1));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.1);
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -1115,23 +995,20 @@ GST_START_TEST (test_peak_track_album)
 
   set_playing_state (element);
 
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_mono (8000, 1024, 1.0));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 1.0);
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   g_object_set (element, "num-tracks", 1, NULL);
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_mono (8000, 1024, 0.5));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.5);
   fail_unless_album_peak (tag_list, 0.5);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 0);
 
   cleanup_rganalysis (element);
@@ -1152,25 +1029,22 @@ GST_START_TEST (test_peak_album_abort_to_track)
   g_object_set (element, "num-tracks", 2, NULL);
   set_playing_state (element);
 
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 1.0, 0.0));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 1.0);
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 1);
 
   g_object_set (element, "num-tracks", 0, NULL);
 
-  send_flush_events (element);
-  send_segment_event (element);
   push_buffer (test_buffer_const_float_stereo (8000, 1024, 0.0, 0.5));
   send_eos_event (element);
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.5);
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -1189,7 +1063,7 @@ GST_START_TEST (test_gain_album)
 
   /* The three tracks are constructed such that if any of these is in fact
    * ignored for the album gain, the album gain will differ. */
-  send_segment_event (element);
+
   accumulator = 0;
   for (i = 8; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
@@ -1199,10 +1073,8 @@ GST_START_TEST (test_gain_album)
   fail_unless_track_peak (tag_list, 0.75);
   fail_unless_track_gain (tag_list, -15.70);
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
-  send_flush_events (element);
-  send_segment_event (element);
   accumulator = 0;
   for (i = 12; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
@@ -1212,10 +1084,8 @@ GST_START_TEST (test_gain_album)
   fail_unless_track_peak (tag_list, 0.5);
   fail_unless_track_gain (tag_list, -12.22);
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
-  send_flush_events (element);
-  send_segment_event (element);
   accumulator = 0;
   for (i = 180; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
@@ -1229,7 +1099,7 @@ GST_START_TEST (test_gain_album)
   /* Strangely, wavegain reports -12.17 for the album, but the fixed
    * metaflac agrees to us.  Could be a 32767 vs. 32768 issue. */
   fail_unless_album_gain (tag_list, -12.18);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -1248,7 +1118,6 @@ GST_START_TEST (test_forced)
   g_object_set (element, "forced", FALSE, NULL);
   set_playing_state (element);
 
-  send_segment_event (element);
   tag_list = gst_tag_list_new_empty ();
   /* Provided values are totally arbitrary. */
   gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
@@ -1262,8 +1131,7 @@ GST_START_TEST (test_forced)
   poll_eos (element);
 
   /* Now back to a track without tags. */
-  send_flush_events (element);
-  send_segment_event (element);
+
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.25, 0.25));
@@ -1271,7 +1139,7 @@ GST_START_TEST (test_forced)
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (44100));
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -1290,7 +1158,6 @@ GST_START_TEST (test_forced_separate)
   g_object_set (element, "forced", FALSE, NULL);
   set_playing_state (element);
 
-  send_segment_event (element);
   tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND, GST_TAG_TRACK_GAIN, 2.21,
       NULL);
@@ -1309,8 +1176,7 @@ GST_START_TEST (test_forced_separate)
   poll_eos (element);
 
   /* Now a track without tags. */
-  send_flush_events (element);
-  send_segment_event (element);
+
   accumulator = 0;
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
@@ -1320,7 +1186,7 @@ GST_START_TEST (test_forced_separate)
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (44100));
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -1343,7 +1209,6 @@ GST_START_TEST (test_forced_after_data)
   g_object_set (element, "forced", FALSE, NULL);
   set_playing_state (element);
 
-  send_segment_event (element);
   for (i = 20; i--;)
     push_buffer (test_buffer_const_float_stereo (8000, 512, 0.5, 0.5));
 
@@ -1355,8 +1220,6 @@ GST_START_TEST (test_forced_after_data)
   send_eos_event (element);
   poll_eos (element);
 
-  send_flush_events (element);
-  send_segment_event (element);
   /* Now back to a normal track, this one has no tags: */
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 8000, 512, 0.25,
@@ -1365,7 +1228,7 @@ GST_START_TEST (test_forced_after_data)
   tag_list = poll_tags (element);
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (8000));
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -1385,7 +1248,6 @@ GST_START_TEST (test_forced_album)
   g_object_set (element, "forced", FALSE, NULL);
   set_playing_state (element);
 
-  send_segment_event (element);
   tag_list = gst_tag_list_new_empty ();
   /* Provided values are totally arbitrary. */
   gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
@@ -1403,8 +1265,6 @@ GST_START_TEST (test_forced_album)
   /* Now an album without tags. */
   g_object_set (element, "num-tracks", 2, NULL);
 
-  send_flush_events (element);
-  send_segment_event (element);
   accumulator = 0;
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
@@ -1414,11 +1274,9 @@ GST_START_TEST (test_forced_album)
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (44100));
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 1);
 
-  send_flush_events (element);
-  send_segment_event (element);
   accumulator = 0;
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
@@ -1429,7 +1287,7 @@ GST_START_TEST (test_forced_album)
   fail_unless_track_gain (tag_list, get_expected_gain (44100));
   fail_unless_album_peak (tag_list, 0.25);
   fail_unless_album_gain (tag_list, get_expected_gain (44100));
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 0);
 
   cleanup_rganalysis (element);
@@ -1447,7 +1305,6 @@ GST_START_TEST (test_forced_album_skip)
   g_object_set (element, "forced", FALSE, "num-tracks", 2, NULL);
   set_playing_state (element);
 
-  send_segment_event (element);
   tag_list = gst_tag_list_new_empty ();
   /* Provided values are totally arbitrary. */
   gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
@@ -1464,8 +1321,6 @@ GST_START_TEST (test_forced_album_skip)
 
   /* This track has no tags, but needs to be skipped anyways since we
    * are in album processing mode. */
-  send_flush_events (element);
-  send_segment_event (element);
   for (i = 20; i--;)
     push_buffer (test_buffer_const_float_stereo (8000, 512, 0.0, 0.0));
   send_eos_event (element);
@@ -1473,8 +1328,6 @@ GST_START_TEST (test_forced_album_skip)
   fail_unless_num_tracks (element, 0);
 
   /* Normal track after the album.  Of course not to be skipped. */
-  send_flush_events (element);
-  send_segment_event (element);
   accumulator = 0;
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 8000, 512, 0.25,
@@ -1484,7 +1337,7 @@ GST_START_TEST (test_forced_album_skip)
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (8000));
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -1501,7 +1354,6 @@ GST_START_TEST (test_forced_album_no_skip)
   g_object_set (element, "forced", FALSE, "num-tracks", 2, NULL);
   set_playing_state (element);
 
-  send_segment_event (element);
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 8000, 512, 0.25,
             0.25));
@@ -1510,13 +1362,11 @@ GST_START_TEST (test_forced_album_no_skip)
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (8000));
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 1);
 
   /* The second track has indeed full tags, but although being not forced, this
    * one has to be processed because album processing is on. */
-  send_flush_events (element);
-  send_segment_event (element);
   tag_list = gst_tag_list_new_empty ();
   /* Provided values are totally arbitrary. */
   gst_tag_list_add (tag_list, GST_TAG_MERGE_APPEND,
@@ -1535,7 +1385,7 @@ GST_START_TEST (test_forced_album_no_skip)
   /* Statistical processing leads to the second track being
    * ignored for the gain (because it is so short): */
   fail_unless_album_gain (tag_list, get_expected_gain (8000));
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 0);
 
   cleanup_rganalysis (element);
@@ -1553,7 +1403,6 @@ GST_START_TEST (test_forced_abort_album_no_skip)
   g_object_set (element, "forced", FALSE, "num-tracks", 2, NULL);
   set_playing_state (element);
 
-  send_segment_event (element);
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 8000, 512, 0.25,
             0.25));
@@ -1562,15 +1411,12 @@ GST_START_TEST (test_forced_abort_album_no_skip)
   fail_unless_track_peak (tag_list, 0.25);
   fail_unless_track_gain (tag_list, get_expected_gain (8000));
   fail_if_album_tags (tag_list);
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
   fail_unless_num_tracks (element, 1);
 
   /* Disabling album processing before end of album: */
   g_object_set (element, "num-tracks", 0, NULL);
 
-  send_flush_events (element);
-  send_segment_event (element);
-
   /* Processing a track that has to be skipped. */
   tag_list = gst_tag_list_new_empty ();
   /* Provided values are totally arbitrary. */
@@ -1598,7 +1444,6 @@ GST_START_TEST (test_reference_level)
 
   set_playing_state (element);
 
-  send_segment_event (element);
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.25, 0.25));
@@ -1610,12 +1455,10 @@ GST_START_TEST (test_reference_level)
   fail_unless (gst_tag_list_get_double (tag_list, GST_TAG_REFERENCE_LEVEL,
           &ref_level) && MATCH_GAIN (ref_level, 89.),
       "Incorrect reference level tag");
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   g_object_set (element, "reference-level", 83., "num-tracks", 2, NULL);
 
-  send_flush_events (element);
-  send_segment_event (element);
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
             0.25, 0.25));
@@ -1627,10 +1470,8 @@ GST_START_TEST (test_reference_level)
   fail_unless (gst_tag_list_get_double (tag_list, GST_TAG_REFERENCE_LEVEL,
           &ref_level) && MATCH_GAIN (ref_level, 83.),
       "Incorrect reference level tag");
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
-  send_flush_events (element);
-  send_segment_event (element);
   accumulator = 0;
   for (i = 20; i--;)
     push_buffer (test_buffer_square_float_stereo (&accumulator, 44100, 512,
@@ -1646,7 +1487,7 @@ GST_START_TEST (test_reference_level)
   fail_unless (gst_tag_list_get_double (tag_list, GST_TAG_REFERENCE_LEVEL,
           &ref_level) && MATCH_GAIN (ref_level, 83.),
       "Incorrect reference level tag");
-  gst_tag_list_unref (tag_list);
+  gst_tag_list_free (tag_list);
 
   cleanup_rganalysis (element);
 }
@@ -1662,8 +1503,6 @@ GST_START_TEST (test_all_formats)
 
   set_playing_state (element);
   for (i = G_N_ELEMENTS (supported_rates); i--;) {
-    send_flush_events (element);
-    send_segment_event (element);
     accumulator = 0;
     for (j = 0; j < 4; j++)
       push_buffer (test_buffer_square_float_stereo (&accumulator,
@@ -1681,7 +1520,7 @@ GST_START_TEST (test_all_formats)
     tag_list = poll_tags (element);
     fail_unless_track_peak (tag_list, 0.25);
     fail_unless_track_gain (tag_list, supported_rates[i].gain);
-    gst_tag_list_unref (tag_list);
+    gst_tag_list_free (tag_list);
   }
 
   cleanup_rganalysis (element);
@@ -1712,7 +1551,7 @@ GST_END_TEST;
   fail_unless_track_peak (tag_list, 0.25);                            \
   fail_unless_track_gain (tag_list,                                   \
       get_expected_gain (sample_rate));                               \
-  gst_tag_list_unref (tag_list);                                       \
+  gst_tag_list_free (tag_list);                                       \
                                                                       \
   cleanup_rganalysis (element);                                       \
 }                                                                     \
@@ -1737,7 +1576,7 @@ GST_END_TEST;
   fail_unless_track_peak (tag_list, 0.25);                            \
   fail_unless_track_gain (tag_list,                                   \
       get_expected_gain (sample_rate));                               \
-  gst_tag_list_unref (tag_list);                                       \
+  gst_tag_list_free (tag_list);                                       \
                                                                       \
   cleanup_rganalysis (element);                                       \
 }                                                                     \
@@ -1763,7 +1602,7 @@ GST_END_TEST;
   fail_unless_track_peak (tag_list, 0.25);                            \
   fail_unless_track_gain (tag_list,                                   \
       get_expected_gain (sample_rate));                               \
-  gst_tag_list_unref (tag_list);                                       \
+  gst_tag_list_free (tag_list);                                       \
                                                                       \
   cleanup_rganalysis (element);                                       \
 }                                                                     \
@@ -1789,7 +1628,7 @@ GST_END_TEST;
   fail_unless_track_peak (tag_list, 0.25);                            \
   fail_unless_track_gain (tag_list,                                   \
       get_expected_gain (sample_rate));                               \
-  gst_tag_list_unref (tag_list);                                       \
+  gst_tag_list_free (tag_list);                                       \
                                                                       \
   cleanup_rganalysis (element);                                       \
 }                                                                     \
