@@ -21,6 +21,7 @@
  */
 
 #include <gst/check/gstcheck.h>
+#include <gst/audio/audio.h>
 
 #include <math.h>
 
@@ -31,11 +32,11 @@ GList *events = NULL;
  * then remove references in every test function */
 static GstPad *mysrcpad, *mysinkpad;
 
-#define RG_VOLUME_CAPS_TEMPLATE_STRING    \
-  "audio/x-raw-float, "                   \
-  "width = (int) 32, "                    \
-  "endianness = (int) BYTE_ORDER, "       \
-  "channels = (int) [ 1, MAX ], "         \
+#define RG_VOLUME_CAPS_TEMPLATE_STRING        \
+  "audio/x-raw, "                             \
+  "format = (string) "GST_AUDIO_NE (F32) ", " \
+  "layout = (string) interleaved, "           \
+  "channels = (int) [ 1, MAX ], "             \
   "rate = (int) [ 1, MAX ]"
 
 static GstStaticPadTemplate sinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
@@ -54,9 +55,16 @@ static GstBuffer *test_buffer_new (gfloat value);
 /* gstcheck sets up a chain function that appends buffers to a global list.
  * This is our equivalent of that for event handling. */
 static gboolean
-event_func (GstPad * pad, GstEvent * event)
+event_func (GstPad * pad, GstObject * parent, GstEvent * event)
 {
-  events = g_list_append (events, event);
+  GST_DEBUG ("received event %p", event);
+  /* not interested in caps event */
+  if (GST_EVENT_TYPE (event) == GST_EVENT_CAPS) {
+    GST_DEBUG ("dropping caps event");
+    gst_event_unref (event);
+  } else {
+    events = g_list_append (events, event);
+  }
 
   return TRUE;
 }
@@ -65,11 +73,12 @@ static GstElement *
 setup_rgvolume (void)
 {
   GstElement *element;
+  GstCaps *caps;
 
   GST_DEBUG ("setup_rgvolume");
   element = gst_check_setup_element ("rgvolume");
-  mysrcpad = gst_check_setup_src_pad (element, &srctemplate, NULL);
-  mysinkpad = gst_check_setup_sink_pad (element, &sinktemplate, NULL);
+  mysrcpad = gst_check_setup_src_pad (element, &srctemplate);
+  mysinkpad = gst_check_setup_sink_pad (element, &sinktemplate);
 
   /* Capture events, to test tag filtering behavior: */
   gst_pad_set_event_function (mysinkpad, event_func);
@@ -77,6 +86,11 @@ setup_rgvolume (void)
   gst_pad_set_active (mysrcpad, TRUE);
   gst_pad_set_active (mysinkpad, TRUE);
 
+  caps = gst_caps_from_string ("audio/x-raw, format = F32LE, "
+      "layout = interleaved, rate = 8000, channels = 1");
+  gst_pad_set_caps (mysrcpad, caps);
+  gst_caps_unref (caps);
+
   return element;
 }
 
@@ -85,15 +99,18 @@ send_newsegment_and_empty_buffer (void)
 {
   GstBuffer *buf;
   GstEvent *ev;
+  GstSegment segment;
 
   fail_unless (g_list_length (events) == 0);
 
-  ev = gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME, 0, -1, 0);
+  gst_segment_init (&segment, GST_FORMAT_TIME);
+  ev = gst_event_new_segment (&segment);
   fail_unless (gst_pad_push_event (mysrcpad, ev),
       "Pushing newsegment event failed");
 
+  /* makes caps event */
   buf = test_buffer_new (0.0);
-  GST_BUFFER_SIZE (buf) = 0;
+  gst_buffer_resize (buf, 0, 0);
   GST_BUFFER_DURATION (buf) = 0;
   GST_BUFFER_OFFSET_END (buf) = GST_BUFFER_OFFSET (buf);
   fail_unless (gst_pad_push (mysrcpad, buf) == GST_FLOW_OK);
@@ -102,6 +119,7 @@ send_newsegment_and_empty_buffer (void)
   fail_unless (events->data == ev);
   gst_mini_object_unref ((GstMiniObject *) events->data);
   events = g_list_remove (events, ev);
+  fail_unless (g_list_length (events) == 0);
 
   fail_unless (g_list_length (buffers) == 1);
   fail_unless (buffers->data == buf);
@@ -150,6 +168,8 @@ send_eos_event (GstElement * element)
 {
   GstEvent *event = gst_event_new_eos ();
 
+  GST_DEBUG ("events : %d", g_list_length (events));
+
   fail_unless (g_list_length (events) == 0);
   fail_unless (gst_pad_push_event (mysrcpad, event),
       "Pushing EOS event failed");
@@ -199,19 +219,17 @@ static GstBuffer *
 test_buffer_new (gfloat value)
 {
   GstBuffer *buf;
-  GstCaps *caps;
+  GstMapInfo map;
   gfloat *data;
   gint i;
 
   buf = gst_buffer_new_and_alloc (8 * sizeof (gfloat));
-  data = (gfloat *) GST_BUFFER_DATA (buf);
+  gst_buffer_map (buf, &map, GST_MAP_WRITE);
+  data = (gfloat *) map.data;
   for (i = 0; i < 8; i++)
     data[i] = value;
+  gst_buffer_unmap (buf, &map);
 
-  caps = gst_caps_from_string ("audio/x-raw-float, "
-      "rate = 8000, channels = 1, endianness = BYTE_ORDER, width = 32");
-  gst_buffer_set_caps (buf, caps);
-  gst_caps_unref (caps);
 
   ASSERT_BUFFER_REFCOUNT (buf, "buf", 1);
 
@@ -235,10 +253,12 @@ static void
 fail_unless_result_gain (GstElement * element, gdouble expected_gain)
 {
   GstBuffer *input_buf, *output_buf;
+  gfloat *data;
   gfloat input_sample, output_sample;
   gdouble gain, prop_gain;
   gboolean is_passthrough, expect_passthrough;
   gint i;
+  GstMapInfo map;
 
   fail_unless (g_list_length (buffers) == 0);
 
@@ -258,16 +278,19 @@ fail_unless_result_gain (GstElement * element, gdouble expected_gain)
 
   buffers = g_list_remove (buffers, output_buf);
   ASSERT_BUFFER_REFCOUNT (output_buf, "output_buf", 1);
-  fail_unless_equals_int (GST_BUFFER_SIZE (output_buf), 8 * sizeof (gfloat));
 
-  output_sample = *((gfloat *) GST_BUFFER_DATA (output_buf));
+  fail_unless_equals_int (gst_buffer_get_size (output_buf),
+      8 * sizeof (gfloat));
 
+  gst_buffer_map (output_buf, &map, GST_MAP_READ);
+  data = (gfloat *) map.data;
+
+  output_sample = *data;
   fail_if (output_sample == 0.0, "First output sample is zero");
   for (i = 1; i < 8; i++) {
-    gfloat output = ((gfloat *) GST_BUFFER_DATA (output_buf))[i];
-
-    fail_unless (output_sample == output, "Output samples not uniform");
+    fail_unless (output_sample == data[i], "Output samples not uniform");
   };
+  gst_buffer_unmap (output_buf, &map);
 
   gain = 20. * log10 (output_sample / input_sample);
   fail_unless (MATCH_GAIN (gain, expected_gain),
@@ -321,15 +344,13 @@ GST_START_TEST (test_events)
 
   send_newsegment_and_empty_buffer ();
 
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_TRACK_GAIN, +4.95, GST_TAG_TRACK_PEAK, 0.59463,
       GST_TAG_ALBUM_GAIN, -1.54, GST_TAG_ALBUM_PEAK, 0.693415,
       GST_TAG_ARTIST, "Foobar", NULL);
   event = gst_event_new_tag (tag_list);
   new_event = send_tag_event (element, event);
-  /* Expect the element to modify the writable event. */
-  fail_unless (event == new_event, "Writable tag event not reused");
   gst_event_parse_tag (new_event, &tag_list);
   fail_unless (gst_tag_list_get_string (tag_list, GST_TAG_ARTIST, &artist));
   fail_unless (g_str_equal (artist, "Foobar"));
@@ -338,21 +359,17 @@ GST_START_TEST (test_events)
 
   /* Same as above, but with a non-writable event. */
 
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_TRACK_GAIN, +4.95, GST_TAG_TRACK_PEAK, 0.59463,
       GST_TAG_ALBUM_GAIN, -1.54, GST_TAG_ALBUM_PEAK, 0.693415,
       GST_TAG_ARTIST, "Foobar", NULL);
   event = gst_event_new_tag (tag_list);
-  /* Holding an extra ref makes the event unwritable: */
-  gst_event_ref (event);
   new_event = send_tag_event (element, event);
-  fail_unless (event != new_event, "Unwritable tag event reused");
   gst_event_parse_tag (new_event, &tag_list);
   fail_unless (gst_tag_list_get_string (tag_list, GST_TAG_ARTIST, &artist));
   fail_unless (g_str_equal (artist, "Foobar"));
   g_free (artist);
-  gst_event_unref (event);
   gst_event_unref (new_event);
 
   cleanup_rgvolume (element);
@@ -371,7 +388,7 @@ GST_START_TEST (test_simple)
 
   send_newsegment_and_empty_buffer ();
 
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_TRACK_GAIN, -3.45, GST_TAG_TRACK_PEAK, 1.0,
       GST_TAG_ALBUM_GAIN, +2.09, GST_TAG_ALBUM_PEAK, 1.0, NULL);
@@ -381,7 +398,7 @@ GST_START_TEST (test_simple)
 
   g_object_set (element, "album-mode", TRUE, NULL);
 
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_TRACK_GAIN, -3.45, GST_TAG_TRACK_PEAK, 1.0,
       GST_TAG_ALBUM_GAIN, +2.09, GST_TAG_ALBUM_PEAK, 1.0, NULL);
@@ -413,7 +430,7 @@ GST_START_TEST (test_fallback_gain)
 
   send_newsegment_and_empty_buffer ();
 
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_TRACK_GAIN, +3.5, GST_TAG_TRACK_PEAK, 1.0,
       GST_TAG_ALBUM_GAIN, -0.5, GST_TAG_ALBUM_PEAK, 1.0, NULL);
@@ -458,7 +475,7 @@ GST_START_TEST (test_fallback_track)
 
   send_newsegment_and_empty_buffer ();
 
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_TRACK_GAIN, +2.11, GST_TAG_TRACK_PEAK, 1.0, NULL);
   fail_unless (send_tag_event (element, gst_event_new_tag (tag_list)) == NULL);
@@ -485,7 +502,7 @@ GST_START_TEST (test_fallback_album)
 
   send_newsegment_and_empty_buffer ();
 
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_ALBUM_GAIN, +3.73, GST_TAG_ALBUM_PEAK, 1.0, NULL);
   fail_unless (send_tag_event (element, gst_event_new_tag (tag_list)) == NULL);
@@ -509,7 +526,7 @@ GST_START_TEST (test_headroom)
 
   send_newsegment_and_empty_buffer ();
 
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_TRACK_GAIN, +3.50, GST_TAG_TRACK_PEAK, 1.0, NULL);
   fail_unless (send_tag_event (element, gst_event_new_tag (tag_list)) == NULL);
@@ -518,7 +535,7 @@ GST_START_TEST (test_headroom)
   send_eos_event (element);
 
   g_object_set (element, "headroom", +2.00, NULL);
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_TRACK_GAIN, +9.18, GST_TAG_TRACK_PEAK, 0.687149, NULL);
   fail_unless (send_tag_event (element, gst_event_new_tag (tag_list)) == NULL);
@@ -528,7 +545,7 @@ GST_START_TEST (test_headroom)
   send_eos_event (element);
 
   g_object_set (element, "album-mode", TRUE, NULL);
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_ALBUM_GAIN, +5.50, GST_TAG_ALBUM_PEAK, 1.0, NULL);
   fail_unless (send_tag_event (element, gst_event_new_tag (tag_list)) == NULL);
@@ -553,7 +570,7 @@ GST_START_TEST (test_reference_level)
 
   send_newsegment_and_empty_buffer ();
 
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_TRACK_GAIN, 0.00, GST_TAG_TRACK_PEAK, 0.2,
       GST_TAG_REFERENCE_LEVEL, 83., NULL);
@@ -566,7 +583,7 @@ GST_START_TEST (test_reference_level)
 
   /* Same as above, but with album gain. */
 
-  tag_list = gst_tag_list_new ();
+  tag_list = gst_tag_list_new_empty ();
   gst_tag_list_add (tag_list, GST_TAG_MERGE_REPLACE,
       GST_TAG_TRACK_GAIN, 1.23, GST_TAG_TRACK_PEAK, 0.1,
       GST_TAG_ALBUM_GAIN, 0.00, GST_TAG_ALBUM_PEAK, 0.2,
